/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 63-3 */

 1 #include <signal.h>
 2 #include <ctype.h>
 3 #include <fcntl.h>
 4 #include <termios.h>
 5 #include "tty_functions.h"      /* Declaration of ttySetCbreak() */
 6 #include "tlpi_hdr.h"
 7
 8 static volatile sig_atomic_t gotSigio = 0;
 9                                 /* Set nonzero on receipt of SIGIO */
10
11 static void
12 sigioHandler(int sig)
13 {
14     gotSigio = 1;
15 }
16
17 int
18 main(int argc, char *argv[])
19 {
20     int flags, j, cnt;
21     struct termios origTermios;
22     char ch;
23     struct sigaction sa;
24     Boolean done;
25
26     /* Establish handler for "I/O possible" signal */
27
28     sigemptyset(&sa.sa_mask);
29     sa.sa_flags = SA_RESTART;
30     sa.sa_handler = sigioHandler;
31     if (sigaction(SIGIO, &sa, NULL) == -1)
32         errExit("sigaction");
33
34     /* Set owner process that is to receive "I/O possible" signal */
35
36     if (fcntl(STDIN_FILENO, F_SETOWN, getpid()) == -1)
37         errExit("fcntl(F_SETOWN)");
38
39     /* Enable "I/O possible" signaling and make I/O nonblocking
40        for file descriptor */
41
42     flags = fcntl(STDIN_FILENO, F_GETFL);
43     if (fcntl(STDIN_FILENO, F_SETFL, flags | O_ASYNC | O_NONBLOCK) == -1)
44         errExit("fcntl(F_SETFL)");
45
46     /* Place terminal in cbreak mode */
47
48     if (ttySetCbreak(STDIN_FILENO, &origTermios) == -1)
49         errExit("ttySetCbreak");
50
51     for (done = FALSE, cnt = 0; !done ; cnt++) {
52         for (j = 0; j < 100000000; j++)
53             continue;                   /* Slow main loop down a little */
54
55         if (gotSigio) {                 /* Is input available? */
56             gotSigio = 0;
57
58             /* Read all available input until error (probably EAGAIN)
59                or EOF (not actually possible in cbreak mode) or a
60                hash (#) character is read */
61
62             while (read(STDIN_FILENO, &ch, 1) > 0 && !done) {
63                 printf("cnt=%d; read %c\n", cnt, ch);
64                 done = ch == '#';
65             }
66         }
67     }
68
69     /* Restore original terminal settings */
70
71     if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &origTermios) == -1)
72         errExit("tcsetattr");
73     exit(EXIT_SUCCESS);
74 }
