/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 63-5 */

 1 #include <sys/epoll.h>
 2 #include <fcntl.h>
 3 #include "tlpi_hdr.h"
 4
 5 #define MAX_BUF     1000        /* Maximum bytes fetched by a single read() */
 6 #define MAX_EVENTS     5        /* Maximum number of events to be returned from
 7                                    a single epoll_wait() call */
 8
 9 int
10 main(int argc, char *argv[])
11 {
12     int epfd, ready, fd, s, j, numOpenFds;
13     struct epoll_event ev;
14     struct epoll_event evlist[MAX_EVENTS];
15     char buf[MAX_BUF];
16
17     if (argc < 2 || strcmp(argv[1], "--help") == 0)
18         usageErr("%s file...\n", argv[0]);
19
20     epfd = epoll_create(argc - 1);
21     if (epfd == -1)
22         errExit("epoll_create");
23
24     /* Open each file on command line, and add it to the "interest
25        list" for the epoll instance */
26
27     for (j = 1; j < argc; j++) {
28         fd = open(argv[j], O_RDONLY);
29         if (fd == -1)
30             errExit("open");
31         printf("Opened \"%s\" on fd %d\n", argv[j], fd);
32
33         ev.events = EPOLLIN;            /* Only interested in input events */
34         ev.data.fd = fd;
35         if (epoll_ctl(epfd, EPOLL_CTL_ADD, fd, &ev) == -1)
36             errExit("epoll_ctl");
37     }
38
39     numOpenFds = argc - 1;
40
41     while (numOpenFds > 0) {
42
43         /* Fetch up to MAX_EVENTS items from the ready list of the
44            epoll instance */
45
46         printf("About to epoll_wait()\n");
47         ready = epoll_wait(epfd, evlist, MAX_EVENTS, -1);
48         if (ready == -1) {
49             if (errno == EINTR)
50                 continue;               /* Restart if interrupted by signal */
51             else
52                 errExit("epoll_wait");
53         }
54
55         printf("Ready: %d\n", ready);
56
57         /* Deal with returned list of events */
58
59         for (j = 0; j < ready; j++) {
60             printf("  fd=%d; events: %s%s%s\n", evlist[j].data.fd,
61                     (evlist[j].events & EPOLLIN)  ? "EPOLLIN "  : "",
62                     (evlist[j].events & EPOLLHUP) ? "EPOLLHUP " : "",
63                     (evlist[j].events & EPOLLERR) ? "EPOLLERR " : "");
64
65             if (evlist[j].events & EPOLLIN) {
66                 s = read(evlist[j].data.fd, buf, MAX_BUF);
67                 if (s == -1)
68                     errExit("read");
69                 printf("    read %d bytes: %.*s\n", s, s, buf);
70
71             } else if (evlist[j].events & (EPOLLHUP | EPOLLERR)) {
72
73                 /* After the epoll_wait(), EPOLLIN and EPOLLHUP may both have
74                    been set. But we'll only get here, and thus close the file
75                    descriptor, if EPOLLIN was not set. This ensures that all
76                    outstanding input (possibly more than MAX_BUF bytes) is
77                    consumed (by further loop iterations) before the file
78                    descriptor is closed. */
79
80                 printf("    closing fd %d\n", evlist[j].data.fd);
81                 if (close(evlist[j].data.fd) == -1)
82                     errExit("close");
83                 numOpenFds--;
84             }
85         }
86     }
87
88     printf("All file descriptors closed; bye\n");
89     exit(EXIT_SUCCESS);
90 }
