/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 63-2 */

 1 #include <time.h>
 2 #include <poll.h>
 3 #include "tlpi_hdr.h"
 4
 5 int
 6 main(int argc, char *argv[])
 7 {
 8     int numPipes, j, ready, randPipe, numWrites;
 9     int (*pfds)[2];                     /* File descriptors for all pipes */
10     struct pollfd *pollFd;
11
12     if (argc < 2 || strcmp(argv[1], "--help") == 0)
13         usageErr("%s num-pipes [num-writes]\n", argv[0]);
14
15     /* Allocate the arrays that we use. The arrays are sized according
16        to the number of pipes specified on command line */
17
18     numPipes = getInt(argv[1], GN_GT_0, "num-pipes");
19
20     pfds = calloc(numPipes, sizeof(int [2]));
21     if (pfds == NULL)
22         errExit("calloc");
23     pollFd = calloc(numPipes, sizeof(struct pollfd));
24     if (pollFd == NULL)
25         errExit("calloc");
26
27     /* Create the number of pipes specified on command line */
28
29     for (j = 0; j < numPipes; j++)
30         if (pipe(pfds[j]) == -1)
31             errExit("pipe %d", j);
32
33     /* Perform specified number of writes to random pipes */
34
35     numWrites = (argc > 2) ? getInt(argv[2], GN_GT_0, "num-writes") : 1;
36
37     srandom((int) time(NULL));
38     for (j = 0; j < numWrites; j++) {
39         randPipe = random() % numPipes;
40         printf("Writing to fd: %3d (read fd: %3d)\n",
41                 pfds[randPipe][1], pfds[randPipe][0]);
42         if (write(pfds[randPipe][1], "a", 1) == -1)
43             errExit("write %d", pfds[randPipe][1]);
44     }
45
46     /* Build the file descriptor list to be supplied to poll(). This list
47        is set to contain the file descriptors for the read ends of all of
48        the pipes. */
49
50     for (j = 0; j < numPipes; j++) {
51         pollFd[j].fd = pfds[j][0];
52         pollFd[j].events = POLLIN;
53     }
54
55     ready = poll(pollFd, numPipes, -1);
56     if (ready == -1)
57         errExit("poll");
58
59     printf("poll() returned: %d\n", ready);
60
61     /* Check which pipes have data available for reading */
62
63     for (j = 0; j < numPipes; j++)
64         if (pollFd[j].revents & POLLIN)
65             printf("Readable: %3d\n", pollFd[j].fd);
66
67     exit(EXIT_SUCCESS);
68 }
