/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 63-1 */

 1 #include <sys/time.h>
 2 #include <sys/select.h>
 3 #include "tlpi_hdr.h"
 4
 5 static void
 6 usageError(const char *progName)
 7 {
 8     fprintf(stderr, "Usage: %s {timeout|-} fd-num[rw]...\n", progName);
 9     fprintf(stderr, "    - means infinite timeout; \n");
10     fprintf(stderr, "    r = monitor for read\n");
11     fprintf(stderr, "    w = monitor for write\n\n");
12     fprintf(stderr, "    e.g.: %s - 0rw 1w\n", progName);
13     exit(EXIT_FAILURE);
14 }
15
16 int
17 main(int argc, char *argv[])
18 {
19     fd_set readfds, writefds;
20     int ready, nfds, fd, numRead, j;
21     struct timeval timeout;
22     struct timeval *pto;
23     char buf[10];                       /* Large enough to hold "rw\0" */
24
25     if (argc < 2 || strcmp(argv[1], "--help") == 0)
26         usageError(argv[0]);
27
28     /* Timeout for select() is specified in argv[1] */
29
30     if (strcmp(argv[1], "-") == 0) {
31         pto = NULL;                     /* Infinite timeout */
32     } else {
33         pto = &timeout;
34         timeout.tv_sec = getLong(argv[1], 0, "timeout");
35         timeout.tv_usec = 0;            /* No microseconds */
36     }
37
38     /* Process remaining arguments to build file descriptor sets */
39
40     nfds = 0;
41     FD_ZERO(&readfds);
42     FD_ZERO(&writefds);
43
44     for (j = 2; j < argc; j++) {
45         numRead = sscanf(argv[j], "%d%2[rw]", &fd, buf);
46         if (numRead != 2)
47             usageError(argv[0]);
48         if (fd >= FD_SETSIZE)
49             cmdLineErr("file descriptor exceeds limit (%d)\n", FD_SETSIZE);
50
51         if (fd >= nfds)
52             nfds = fd + 1;              /* Record maximum fd + 1 */
53         if (strchr(buf, 'r') != NULL)
54             FD_SET(fd, &readfds);
55         if (strchr(buf, 'w') != NULL)
56             FD_SET(fd, &writefds);
57     }
58
59     /* We've built all of the arguments; now call select() */
60
61     ready = select(nfds, &readfds, &writefds, NULL, pto);
62                                         /* Ignore exceptional events */
63     if (ready == -1)
64         errExit("select");
65
66     /* Display results of select() */
67
68     printf("ready = %d\n", ready);
69     for (fd = 0; fd < nfds; fd++)
70         printf("%d: %s%s\n", fd, FD_ISSET(fd, &readfds) ? "r" : "",
71                 FD_ISSET(fd, &writefds) ? "w" : "");
72
73     if (pto != NULL)
74         printf("timeout after select(): %ld.%03ld\n",
75                (long) timeout.tv_sec, (long) timeout.tv_usec / 1000);
76     exit(EXIT_SUCCESS);
77 }
