 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 37-2 */
12
13  1 #include <sys/stat.h>
14  2 #include <fcntl.h>
15  3 #include "become_daemon.h"
16  4 #include "tlpi_hdr.h"
17  5
18  6 int                                     /* Returns 0 on success, -1 on error */
19  7 becomeDaemon(int flags)
20  8 {
21  9     int maxfd, fd;
22 10
23 11     switch (fork()) {                   /* Become background process */
24 12     case -1: return -1;
25 13     case 0:  break;                     /* Child falls through... */
26 14     default: _exit(EXIT_SUCCESS);       /* while parent terminates */
27 15     }
28 16
29 17     if (setsid() == -1)                 /* Become leader of new session */
30 18         return -1;
31 19
32 20     switch (fork()) {                   /* Ensure we are not session leader */
33 21     case -1: return -1;
34 22     case 0:  break;
35 23     default: _exit(EXIT_SUCCESS);
36 24     }
37 25
38 26     if (!(flags & BD_NO_UMASK0))
39 27         umask(0);                       /* Clear file mode creation mask */
40 28
41 29     if (!(flags & BD_NO_CHDIR))
42 30         chdir("/");                     /* Change to root directory */
43 31
44 32     if (!(flags & BD_NO_CLOSE_FILES)) { /* Close all open files */
45 33         maxfd = sysconf(_SC_OPEN_MAX);
46 34         if (maxfd == -1)                /* Limit is indeterminate... */
47 35             maxfd = BD_MAX_CLOSE;       /* so take a guess */
48 36
49 37         for (fd = 0; fd < maxfd; fd++)
50 38             close(fd);
51 39     }
52 40
53 41     if (!(flags & BD_NO_REOPEN_STD_FDS)) {
54 42         close(STDIN_FILENO);            /* Reopen standard fd's to /dev/null */
55 43
56 44         fd = open("/dev/null", O_RDWR);
57 45
58 46         if (fd != STDIN_FILENO)         /* 'fd' should be 0 */
59 47             return -1;
60 48         if (dup2(STDIN_FILENO, STDOUT_FILENO) != STDOUT_FILENO)
61 49             return -1;
62 50         if (dup2(STDIN_FILENO, STDERR_FILENO) != STDERR_FILENO)
63 51             return -1;
64 52     }
65 53
66 54     return 0;
67 55 }
