/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 18-7 */

 1 #define _XOPEN_SOURCE 600   /* Get nftw() and S_IFSOCK declarations */
 2 #include <ftw.h>
 3 #include "tlpi_hdr.h"
 4
 5 static int numReg = 0, numDir = 0, numSymLk = 0, numSocket = 0,
 6            numFifo = 0, numChar = 0, numBlock = 0;
 7
 8 static int
 9 countFile(const char *path, const struct stat *sb, int flag, struct FTW *ftwb)
10 {
11     switch (sb->st_mode & S_IFMT) {
12     case S_IFREG:  numReg++;    break;
13     case S_IFDIR:  numDir++;    break;
14     case S_IFCHR:  numChar++;   break;
15     case S_IFBLK:  numBlock++;  break;
16     case S_IFLNK:  numSymLk++;  break;
17     case S_IFIFO:  numFifo++;   break;
18     case S_IFSOCK: numSocket++; break;
19     }
20     return 0;           /* Always tell nftw() to continue */
21 }
22
23 static void
24 printStats(const char *msg, int num, int numFiles)
25 {
26     printf("%-15s   %6d %6.1f%%\n", msg, num, num * 100.0 / numFiles);
27 }
28
29 int
30 main(int argc, char *argv[])
31 {
32     int numFiles;       /* Total number of files */
33
34     if (argc != 2 || strcmp(argv[1], "--help") == 0)
35         usageErr("%s dir-path\n", argv[0]);
36
37     /* Traverse directory tree counting files; don't follow symbolic links */
38
39     if (nftw(argv[1], &countFile, 20, FTW_PHYS) == -1) {
40         perror("nftw");
41         exit(EXIT_FAILURE);
42     }
43
44     numFiles = numReg + numDir + numSymLk + numSocket +
45                 numFifo + numChar + numBlock;
46
47     if (numFiles == 0) {
48         printf("No files found\n");
49     } else {
50         printf("Total files:      %6d\n", numFiles);
51         printStats("Regular:", numReg, numFiles);
52         printStats("Directory:", numDir, numFiles);
53         printStats("Char device:", numChar, numFiles);
54         printStats("Block device:", numBlock, numFiles);
55         printStats("Symbolic link:", numSymLk, numFiles);
56         printStats("FIFO:", numFifo, numFiles);
57         printStats("Socket:", numSocket, numFiles);
58     }
59     exit(EXIT_SUCCESS);
60 }
