/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 18-2 */

 1 #include <dirent.h>
 2 #include "tlpi_hdr.h"
 3
 4 static void             /* List all files in directory 'dirpath' */
 5 listFiles(const char *dirpath)
 6 {
 7     DIR *dirp;
 8     struct dirent *dp;
 9     Boolean isCurrent;          /* True if 'dirpath' is "." */
10
11     isCurrent = strcmp(dirpath, ".") == 0;
12
13     dirp = opendir(dirpath);
14     if (dirp  == NULL) {
15         errMsg("opendir failed on '%s'", dirpath);
16         return;
17     }
18
19     /* For each entry in this directory, print directory + filename */
20
21     for (;;) {
22         errno = 0;              /* To distinguish error from end-of-directory */
23         dp = readdir(dirp);
24         if (dp == NULL)
25             break;
26
27         if (strcmp(dp->d_name, ".") == 0 || strcmp(dp->d_name, "..") == 0)
28             continue;           /* Skip . and .. */
29
30         if (!isCurrent)
31             printf("%s/", dirpath);
32         printf("%s\n", dp->d_name);
33     }
34
35     if (errno != 0)
36         errExit("readdir");
37
38     if (closedir(dirp) == -1)
39         errMsg("closedir");
40 }
41
42 int
43 main(int argc, char *argv[])
44 {
45     if (argc > 1 && strcmp(argv[1], "--help") == 0)
46         usageErr("%s [dir-path...]\n", argv[0]);
47
48     if (argc == 1)              /* No arguments - use current directory */
49         listFiles(".");
50     else
51         for (argv++; *argv; argv++)
52             listFiles(*argv);
53
54     exit(EXIT_SUCCESS);
55 }
