/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 18-3 */

 1 #define _XOPEN_SOURCE 600       /* Get nftw() and S_IFSOCK declarations */
 2 #include <ftw.h>
 3 #include "tlpi_hdr.h"
 4
 5 static void
 6 usageError(const char *progName, const char *msg)
 7 {
 8     if (msg != NULL)
 9         fprintf(stderr, "%s\n", msg);
10     fprintf(stderr, "Usage: %s [-d] [-m] [-p] [directory-path]\n", progName);
11     fprintf(stderr, "\t-d Use FTW_DEPTH flag\n");
12     fprintf(stderr, "\t-m Use FTW_MOUNT flag\n");
13     fprintf(stderr, "\t-p Use FTW_PHYS flag\n");
14     exit(EXIT_FAILURE);
15 }
16
17 static int                      /* Function called by nftw() */
18 dirTree(const char *pathname, const struct stat *sbuf, int type,
19         struct FTW *ftwb)
20 {
21     switch (sbuf->st_mode & S_IFMT) {       /* Print file type */
22     case S_IFREG:  printf("-"); break;
23     case S_IFDIR:  printf("d"); break;
24     case S_IFCHR:  printf("c"); break;
25     case S_IFBLK:  printf("b"); break;
26     case S_IFLNK:  printf("l"); break;
27     case S_IFIFO:  printf("p"); break;
28     case S_IFSOCK: printf("s"); break;
29     default:       printf("?"); break;      /* Should never happen (on Linux) */
30     }
31
32     printf(" %s  ",
33             (type == FTW_D)  ? "D  " : (type == FTW_DNR) ? "DNR" :
34             (type == FTW_DP) ? "DP " : (type == FTW_F)   ? "F  " :
35             (type == FTW_SL) ? "SL " : (type == FTW_SLN) ? "SLN" :
36             (type == FTW_NS) ? "NS " : "  ");
37
38     if (type != FTW_NS)
39         printf("%7ld ", (long) sbuf->st_ino);
40     else
41         printf("        ");
42
43     printf(" %*s", 4 * ftwb->level, "");        /* Indent suitably */
44     printf("%s\n",  &pathname[ftwb->base]);     /* Print basename */
45     return 0;                                   /* Tell nftw() to continue */
46 }
47
48 int
49 main(int argc, char *argv[])
50 {
51     int flags, opt;
52
53     flags = 0;
54     while ((opt = getopt(argc, argv, "dmp")) != -1) {
55         switch (opt) {
56         case 'd': flags |= FTW_DEPTH;   break;
57         case 'm': flags |= FTW_MOUNT;   break;
58         case 'p': flags |= FTW_PHYS;    break;
59         default:  usageError(argv[0], NULL);
60         }
61     }
62
63     if (argc > optind + 1)
64         usageError(argv[0], NULL);
65
66     if (nftw((argc > optind) ? argv[optind] : ".", dirTree, 10, flags) == -1) {
67         perror("nftw");
68         exit(EXIT_FAILURE);
69     }
70     exit(EXIT_SUCCESS);
71 }
