/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 13 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include "tlpi_hdr.h"
 4
 5 int
 6 main(int argc, char *argv[])
 7 {
 8     size_t bufSize, numBytes, thisWrite, totWritten;
 9     char *buf;
10     int fd, openFlags;
11
12     if (argc != 4 || strcmp(argv[1], "--help") == 0)
13         usageErr("%s file num-bytes buf-size\n", argv[0]);
14
15     numBytes = getLong(argv[2], GN_GT_0, "num-bytes");
16     bufSize = getLong(argv[3], GN_GT_0, "buf-size");
17
18     buf = malloc(bufSize);
19     if (buf == NULL)
20         errExit("malloc");
21
22     openFlags = O_CREAT | O_WRONLY;
23
24 #if defined(USE_O_SYNC) && defined(O_SYNC)
25     openFlags |= O_SYNC;
26 #endif
27
28     fd = open(argv[1], openFlags, S_IRUSR | S_IWUSR);
29     if (fd == -1)
30         errExit("open");
31
32     for (totWritten = 0; totWritten < numBytes;
33             totWritten += thisWrite) {
34         thisWrite = min(bufSize, numBytes - totWritten);
35
36         if (write(fd, buf, thisWrite) != thisWrite)
37             fatal("partial/failed write");
38
39 #ifdef USE_FSYNC
40         if (fsync(fd))
41             errExit("fsync");
42 #endif
43 #ifdef USE_FDATASYNC
44         if (fdatasync(fd))
45             errExit("fdatasync");
46 #endif
47     }
48
49     if (close(fd) == -1)
50         errExit("close");
51     exit(EXIT_SUCCESS);
52 }
