/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 4-1 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include "tlpi_hdr.h"
 4
 5 #ifndef BUF_SIZE        /* Allow "cc -D" to override definition */
 6 #define BUF_SIZE 1024
 7 #endif
 8
 9 int
10 main(int argc, char *argv[])
11 {
12     int inputFd, outputFd, openFlags;
13     mode_t filePerms;
14     ssize_t numRead;
15     char buf[BUF_SIZE];
16
17     if (argc != 3 || strcmp(argv[1], "--help") == 0)
18         usageErr("%s old-file new-file\n", argv[0]);
19
20     /* Open input and output files */
21
22     inputFd = open(argv[1], O_RDONLY);
23     if (inputFd == -1)
24         errExit("opening file %s", argv[1]);
25
26     openFlags = O_CREAT | O_WRONLY | O_TRUNC;
27     filePerms = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP |
28                 S_IROTH | S_IWOTH;      /* rw-rw-rw- */
29     outputFd = open(argv[2], openFlags, filePerms);
30     if (outputFd == -1)
31         errExit("opening file %s", argv[2]);
32
33     /* Transfer data until we encounter end of input or an error */
34
35     while ((numRead = read(inputFd, buf, BUF_SIZE)) > 0)
36         if (write(outputFd, buf, numRead) != numRead)
37             fatal("couldn't write whole buffer");
38     if (numRead == -1)
39         errExit("read");
40
41     if (close(inputFd) == -1)
42         errExit("close input");
43     if (close(outputFd) == -1)
44         errExit("close output");
45
46     exit(EXIT_SUCCESS);
47 }
