/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 4-3 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include <ctype.h>
 4 #include "tlpi_hdr.h"
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     size_t len;
10     off_t offset;
11     int fd, ap, j;
12     char *buf;
13     ssize_t numRead, numWritten;
14
15     if (argc < 3 || strcmp(argv[1], "--help") == 0)
16         usageErr("%s file {r<length>|R<length>|w<string>|s<offset>}...\n",
17                  argv[0]);
18
19     fd = open(argv[1], O_RDWR | O_CREAT,
20                 S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP |
21                 S_IROTH | S_IWOTH);                     /* rw-rw-rw- */
22     if (fd == -1)
23         errExit("open");
24
25     for (ap = 2; ap < argc; ap++) {
26         switch (argv[ap][0]) {
27         case 'r':   /* Display bytes at current offset, as text */
28         case 'R':   /* Display bytes at current offset, in hex */
29             len = getLong(&argv[ap][1], GN_ANY_BASE, argv[ap]);
30
31             buf = malloc(len);
32             if (buf == NULL)
33                 errExit("malloc");
34
35             numRead = read(fd, buf, len);
36             if (numRead == -1)
37                 errExit("read");
38
39             if (numRead == 0) {
40                 printf("%s: end-of-file\n", argv[ap]);
41             } else {
42                 printf("%s: ", argv[ap]);
43                 for (j = 0; j < numRead; j++) {
44                     if (argv[ap][0] == 'r')
45                         printf("%c", isprint((unsigned char) buf[j]) ?
46                                                 buf[j] : '?');
47                     else
48                         printf("%02x ", (unsigned int) buf[j]);
49                 }
50                 printf("\n");
51             }
52
53             free(buf);
54             break;
55
56         case 'w':   /* Write string at current offset */
57             numWritten = write(fd, &argv[ap][1], strlen(&argv[ap][1]));
58             if (numWritten == -1)
59                 errExit("write");
60             printf("%s: wrote %ld bytes\n", argv[ap], (long) numWritten);
61             break;
62
63         case 's':   /* Change file offset */
64             offset = getLong(&argv[ap][1], GN_ANY_BASE, argv[ap]);
65             if (lseek(fd, offset, SEEK_SET) == -1)
66                 errExit("lseek");
67             printf("%s: seek succeeded\n", argv[ap]);
68             break;
69
70         default:
71             cmdLineErr("Argument must start with [rRws]: %s\n", argv[ap]);
72         }
73     }
74
75     exit(EXIT_SUCCESS);
76 }
