/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 55-2 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include "tlpi_hdr.h"
 4
 5 #define MAX_LINE 100
 6
 7 static void
 8 displayCmdFmt(void)
 9 {
10     printf("\n    Format: cmd lock start length [whence]\n\n");
11     printf("    'cmd' is 'g' (GETLK), 's' (SETLK), or 'w' (SETLKW)\n");
12     printf("    'lock' is 'r' (READ), 'w' (WRITE), or 'u' (UNLOCK)\n");
13     printf("    'start' and 'length' specify byte range to lock\n");
14     printf("    'whence' is 's' (SEEK_SET, default), 'c' (SEEK_CUR), "
15            "or 'e' (SEEK_END)\n\n");
16 }
17
18 int
19 main(int argc, char *argv[])
20 {
21     int fd, numRead, cmd, status;
22     char lock, cmdCh, whence, line[MAX_LINE];
23     struct flock fl;
24     long long len, st;
25
26     if (argc != 2 || strcmp(argv[1], "--help") == 0)
27         usageErr("%s file\n", argv[0]);
28
29     fd = open(argv[1], O_RDWR);
30     if (fd == -1)
31         errExit("open (%s)", argv[1]);
32
33     printf("Enter ? for help\n");
34
35     for (;;) {          /* Prompt for locking command and carry it out */
36         printf("PID=%ld> ", (long) getpid());
37         fflush(stdout);
38
39         if (fgets(line, MAX_LINE, stdin) == NULL)       /* EOF */
40             exit(EXIT_SUCCESS);
41         line[strlen(line) - 1] = '\0';          /* Remove trailing '\n' */
42
43         if (*line == '\0')
44             continue;                           /* Skip blank lines */
45
46         if (line[0] == '?') {
47             displayCmdFmt();
48             continue;
49         }
50
51         whence = 's';                   /* In case not otherwise filled in */
52
53         numRead = sscanf(line, "%c %c %lld %lld %c", &cmdCh, &lock,
54                         &st, &len, &whence);
55         fl.l_start = st;
56         fl.l_len = len;
57
58         if (numRead < 4 || strchr("gsw", cmdCh) == NULL ||
59                 strchr("rwu", lock) == NULL || strchr("sce", whence) == NULL) {
60             printf("Invalid command!\n");
61             continue;
62         }
63
64         cmd = (cmdCh == 'g') ? F_GETLK : (cmdCh == 's') ? F_SETLK : F_SETLKW;
65         fl.l_type = (lock == 'r') ? F_RDLCK : (lock == 'w') ? F_WRLCK : F_UNLCK;
66         fl.l_whence = (whence == 'c') ? SEEK_CUR :
67                       (whence == 'e') ? SEEK_END : SEEK_SET;
68
69         status = fcntl(fd, cmd, &fl);           /* Perform request... */
70
71         if (cmd == F_GETLK) {                   /* ... and see what happened */
72             if (status == -1) {
73                 errMsg("fcntl - F_GETLK");
74             } else {
75                 if (fl.l_type == F_UNLCK)
76                     printf("[PID=%ld] Lock can be placed\n", (long) getpid());
77                 else                            /* Locked out by someone else */
78                     printf("[PID=%ld] Denied by %s lock on %lld:%lld "
79                             "(held by PID %ld)\n", (long) getpid(),
80                             (fl.l_type == F_RDLCK) ? "READ" : "WRITE",
81                             (long long) fl.l_start,
82                             (long long) fl.l_len, (long) fl.l_pid);
83             }
84         } else {                /* F_SETLK, F_SETLKW */
85             if (status == 0)
86                 printf("[PID=%ld] %s\n", (long) getpid(),
87                         (lock == 'u') ? "unlocked" : "got lock");
88             else if (errno == EAGAIN || errno == EACCES)        /* F_SETLK */
89                 printf("[PID=%ld] failed (incompatible lock)\n",
90                         (long) getpid());
91             else if (errno == EDEADLK)                          /* F_SETLKW */
92                 printf("[PID=%ld] failed (deadlock)\n", (long) getpid());
93             else
94                 errMsg("fcntl - F_SETLK(W)");
95         }
96     }
97 }
