/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 15-1 */

 1 #define _BSD_SOURCE     /* Get major() and minor() from <sys/types.h> */
 2 #include <sys/types.h>
 3 #include <sys/stat.h>
 4 #include <time.h>
 5 #include "file_perms.h"
 6 #include "tlpi_hdr.h"
 7
 8 static void
 9 displayStatInfo(const struct stat *sb)
10 {
11     printf("File type:                ");
12
13     switch (sb->st_mode & S_IFMT) {
14     case S_IFREG:  printf("regular file\n");            break;
15     case S_IFDIR:  printf("directory\n");               break;
16     case S_IFCHR:  printf("character device\n");        break;
17     case S_IFBLK:  printf("block device\n");            break;
18     case S_IFLNK:  printf("symbolic (soft) link\n");    break;
19     case S_IFIFO:  printf("FIFO or pipe\n");            break;
20     case S_IFSOCK: printf("socket\n");                  break;
21     default:       printf("unknown file type?\n");      break;
22     }
23
24     printf("Device containing i-node: major=%ld   minor=%ld\n",
25                 (long) major(sb->st_dev), (long) minor(sb->st_dev));
26
27     printf("I-node number:            %ld\n", (long) sb->st_ino);
28
29     printf("Mode:                     %lo (%s)\n",
30             (unsigned long) sb->st_mode, filePermStr(sb->st_mode, 0));
31
32     if (sb->st_mode & (S_ISUID | S_ISGID | S_ISVTX))
33         printf("    special bits set:     %s%s%s\n",
34                 (sb->st_mode & S_ISUID) ? "set-UID " : "",
35                 (sb->st_mode & S_ISGID) ? "set-GID " : "",
36                 (sb->st_mode & S_ISVTX) ? "sticky " : "");
37
38     printf("Number of (hard) links:   %ld\n", (long) sb->st_nlink);
39
40     printf("Ownership:                UID=%ld   GID=%ld\n",
41             (long) sb->st_uid, (long) sb->st_gid);
42
43     if (S_ISCHR(sb->st_mode) || S_ISBLK(sb->st_mode))
44         printf("Device number (st_rdev):  major=%ld; minor=%ld\n",
45                 (long) major(sb->st_rdev), (long) minor(sb->st_rdev));
46
47     printf("File size:                %lld bytes\n", (long long) sb->st_size);
48     printf("Optimal I/O block size:   %ld bytes\n", (long) sb->st_blksize);
49     printf("512B blocks allocated:    %lld\n", (long long) sb->st_blocks);
50
51     printf("Last file access:         %s", ctime(&sb->st_atime));
52     printf("Last file modification:   %s", ctime(&sb->st_mtime));
53     printf("Last status change:       %s", ctime(&sb->st_ctime));
54 }
55
56 int
57 main(int argc, char *argv[])
58 {
59     struct stat sb;
60     Boolean statLink;           /* True if "-l" specified (i.e., use lstat) */
61     int fname;                  /* Location of filename argument in argv[] */
62
63     statLink = (argc > 1) && strcmp(argv[1], "-l") == 0;
64                                 /* Simple parsing for "-l" */
65     fname = statLink ? 2 : 1;
66
67     if (fname >= argc || (argc > 1 && strcmp(argv[1], "--help") == 0))
68         usageErr("%s [-l] file\n"
69                 "        -l = use lstat() instead of stat()\n", argv[0]);
70
71     if (statLink) {
72         if (lstat(argv[fname], &sb) == -1)
73             errExit("lstat");
74     } else {
75         if (stat(argv[fname], &sb) == -1)
76             errExit("stat");
77     }
78
79     displayStatInfo(&sb);
80
81     exit(EXIT_SUCCESS);
82 }
