/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 15-5 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include "file_perms.h"
 4 #include "tlpi_hdr.h"
 5
 6 #define MYFILE "myfile"
 7 #define MYDIR  "mydir"
 8 #define FILE_PERMS    (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP)
 9 #define DIR_PERMS     (S_IRWXU | S_IRWXG | S_IRWXO)
10 #define UMASK_SETTING (S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH)
11
12 int
13 main(int argc, char *argv[])
14 {
15     int fd;
16     struct stat sb;
17     mode_t u;
18
19     umask(UMASK_SETTING);
20
21     fd = open(MYFILE, O_RDWR | O_CREAT | O_EXCL, FILE_PERMS);
22     if (fd == -1)
23         errExit("open-%s", MYFILE);
24     if (mkdir(MYDIR, DIR_PERMS) == -1)
25         errExit("mkdir-%s", MYDIR);
26
27     u = umask(0);               /* Retrieves (and clears) umask value */
28
29     if (stat(MYFILE, &sb) == -1)
30         errExit("stat-%s", MYFILE);
31     printf("Requested file perms: %s\n", filePermStr(FILE_PERMS, 0));
32     printf("Process umask:        %s\n", filePermStr(u, 0));
33     printf("Actual file perms:    %s\n\n", filePermStr(sb.st_mode, 0));
34
35     if (stat(MYDIR, &sb) == -1)
36         errExit("stat-%s", MYDIR);
37     printf("Requested dir. perms: %s\n", filePermStr(DIR_PERMS, 0));
38     printf("Process umask:        %s\n", filePermStr(u, 0));
39     printf("Actual dir. perms:    %s\n", filePermStr(sb.st_mode, 0));
40
41     if (unlink(MYFILE) == -1)
42         errMsg("unlink-%s", MYFILE);
43     if (rmdir(MYDIR) == -1)
44         errMsg("rmdir-%s", MYDIR);
45     exit(EXIT_SUCCESS);
46 }
