/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing B-1 */

 1 #include <ctype.h>
 2 #include "tlpi_hdr.h"
 3
 4 #define printable(ch) (isprint((unsigned char) ch) ? ch : '#')
 5
 6 static void             /* Print "usage" message and exit */
 7 usageError(char *progName, char *msg, int opt)
 8 {
 9     if (msg != NULL && opt != 0)
10         fprintf(stderr, "%s (-%c)\n", msg, printable(opt));
11     fprintf(stderr, "Usage: %s [-p arg] [-x]\n", progName);
12     exit(EXIT_FAILURE);
13 }
14
15 int
16 main(int argc, char *argv[])
17 {
18     int opt, xfnd;
19     char *pstr;
20
21     xfnd = 0;
22     pstr = NULL;
23
24     while ((opt = getopt(argc, argv, ":p:x")) != -1) {
25         printf("opt =%3d (%c); optind = %d", opt, printable(opt), optind);
26         if (opt == '?' || opt == ':')
27             printf("; optopt =%3d (%c)", optopt, printable(optopt));
28         printf("\n");
29
30         switch (opt) {
31         case 'p': pstr = optarg;        break;
32         case 'x': xfnd++;               break;
33         case ':': usageError(argv[0], "Missing argument", optopt);
34         case '?': usageError(argv[0], "Unrecognized option", optopt);
35         default:  fatal("Unexpected case in switch()");
36         }
37     }
38
39     if (xfnd != 0)
40         printf("-x was specified (count=%d)\n", xfnd);
41     if (pstr != NULL)
42         printf("-p was specified with the value \"%s\"\n", pstr);
43     if (optind < argc)
44         printf("First nonoption argument is \"%s\" at argv[%d]\n",
45                 argv[optind], optind);
46     exit(EXIT_SUCCESS);
47 }
