 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 47-10 */
12
13  1 #include <sys/types.h>
14  2 #include <sys/sem.h>
15  3 #include "semun.h"                      /* Definition of semun union */
16  4 #include "binary_sems.h"
17  5
18  6 Boolean bsUseSemUndo = FALSE;
19  7 Boolean bsRetryOnEintr = TRUE;
20  8
21  9 int                     /* Initialize semaphore to 1 (i.e., "available") */
22 10 initSemAvailable(int semId, int semNum)
23 11 {
24 12     union semun arg;
25 13
26 14     arg.val = 1;
27 15     return semctl(semId, semNum, SETVAL, arg);
28 16 }
29 17
30 18 int                     /* Initialize semaphore to 0 (i.e., "in use") */
31 19 initSemInUse(int semId, int semNum)
32 20 {
33 21     union semun arg;
34 22
35 23     arg.val = 0;
36 24     return semctl(semId, semNum, SETVAL, arg);
37 25 }
38 26
39 27 /* Reserve semaphore (blocking), return 0 on success, or -1 with 'errno'
40 28    set to EINTR if operation was interrupted by a signal handler */
41 29
42 30 int                     /* Reserve semaphore - decrement it by 1 */
43 31 reserveSem(int semId, int semNum)
44 32 {
45 33     struct sembuf sops;
46 34
47 35     sops.sem_num = semNum;
48 36     sops.sem_op = -1;
49 37     sops.sem_flg = bsUseSemUndo ? SEM_UNDO : 0;
50 38
51 39     while (semop(semId, &sops, 1) == -1)
52 40         if (errno != EINTR || !bsRetryOnEintr)
53 41             return -1;
54 42
55 43     return 0;
56 44 }
57 45
58 46 int                     /* Release semaphore - increment it by 1 */
59 47 releaseSem(int semId, int semNum)
60 48 {
61 49     struct sembuf sops;
62 50
63 51     sops.sem_num = semNum;
64 52     sops.sem_op = 1;
65 53     sops.sem_flg = bsUseSemUndo ? SEM_UNDO : 0;
66 54
67 55     return semop(semId, &sops, 1);
68 56 }
