 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 55-4 */
12
13  1 #include <sys/stat.h>
14  2 #include <fcntl.h>
15  3 #include "region_locking.h"             /* For lockRegion() */
16  4 #include "create_pid_file.h"            /* Declares createPidFile() and
17  5                                            defines CPF_CLOEXEC */
18  6 #include "tlpi_hdr.h"
19  7
20  8 #define BUF_SIZE 100            /* Large enough to hold maximum PID as string */
21  9
22 10 /* Open/create the file named in 'pidFile', lock it, optionally set the
23 11    close-on-exec flag for the file descriptor, write our PID into the file,
24 12    and (in case the caller is interested) return the file descriptor
25 13    referring to the locked file. The caller is responsible for deleting
26 14    'pidFile' file (just) before process termination. 'progName' should be the
27 15    name of the calling program (i.e., argv[0] or similar), and is used only for
28 16    diagnostic messages. If we can't open 'pidFile', or we encounter some other
29 17    error, then we print an appropriate diagnostic and terminate. */
30 18
31 19 int
32 20 createPidFile(const char *progName, const char *pidFile, int flags)
33 21 {
34 22     int fd;
35 23     char buf[BUF_SIZE];
36 24
37 25     fd = open(pidFile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
38 26     if (fd == -1)
39 27         errExit("Could not open PID file %s", pidFile);
40 28
41 29     if (flags & CPF_CLOEXEC) {
42 30
43 31         /* Set the close-on-exec file descriptor flag */
44 32
45 33         flags = fcntl(fd, F_GETFD);                     /* Fetch flags */
46 34         if (flags == -1)
47 35             errExit("Could not get flags for PID file %s", pidFile);
48 36
49 37         flags |= FD_CLOEXEC;                            /* Turn on FD_CLOEXEC */
50 38
51 39         if (fcntl(fd, F_SETFD, flags) == -1)            /* Update flags */
52 40             errExit("Could not set flags for PID file %s", pidFile);
53 41     }
54 42
55 43     if (lockRegion(fd, F_WRLCK, SEEK_SET, 0, 0) == -1) {
56 44         if (errno  == EAGAIN || errno == EACCES)
57 45             fatal("PID file '%s' is locked; probably "
58 46                      "'%s' is already running", pidFile, progName);
59 47         else
60 48             errExit("Unable to lock PID file '%s'", pidFile);
61 49     }
62 50
63 51     if (ftruncate(fd, 0) == -1)
64 52         errExit("Could not truncate PID file '%s'", pidFile);
65 53
66 54     snprintf(buf, BUF_SIZE, "%ld\n", (long) getpid());
67 55     if (write(fd, buf, strlen(buf)) != strlen(buf))
68 56         fatal("Writing to PID file '%s'", pidFile);
69 57
70 58     return fd;
71 59 }
