/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Lesser General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
\*************************************************************************/

/* Listing 3-3 */

  1 #include <stdarg.h>
  2 #include "error_functions.h"
  3 #include "tlpi_hdr.h"
  4 #include "ename.c.inc"          /* Defines ename and MAX_ENAME */
  5
  6 #ifdef __GNUC__
  7 __attribute__ ((__noreturn__))
  8 #endif
  9 static void
 10 terminate(Boolean useExit3)
 11 {
 12     char *s;
 13
 14     /* Dump core if EF_DUMPCORE environment variable is defined and
 15        is a nonempty string; otherwise call exit(3) or _exit(2),
 16        depending on the value of 'useExit3'. */
 17
 18     s = getenv("EF_DUMPCORE");
 19
 20     if (s != NULL && *s != '\0')
 21         abort();
 22     else if (useExit3)
 23         exit(EXIT_FAILURE);
 24     else
 25         _exit(EXIT_FAILURE);
 26 }
 27
 28 static void
 29 outputError(Boolean useErr, int err, Boolean flushStdout,
 30         const char *format, va_list ap)
 31 {
 32 #define BUF_SIZE 500
 33     char buf[BUF_SIZE], userMsg[BUF_SIZE], errText[BUF_SIZE];
 34
 35     vsnprintf(userMsg, BUF_SIZE, format, ap);
 36
 37     if (useErr)
 38         snprintf(errText, BUF_SIZE, " [%s %s]",
 39                 (err > 0 && err <= MAX_ENAME) ?
 40                 ename[err] : "?UNKNOWN?", strerror(err));
 41     else
 42         snprintf(errText, BUF_SIZE, ":");
 43
 44     snprintf(buf, BUF_SIZE, "ERROR%s %s\n", errText, userMsg);
 45
 46     if (flushStdout)
 47         fflush(stdout);       /* Flush any pending stdout */
 48     fputs(buf, stderr);
 49     fflush(stderr);           /* In case stderr is not line-buffered */
 50 }
 51
 52 void
 53 errMsg(const char *format, ...)
 54 {
 55     va_list argList;
 56     int savedErrno;
 57
 58     savedErrno = errno;       /* In case we change it here */
 59
 60     va_start(argList, format);
 61     outputError(TRUE, errno, TRUE, format, argList);
 62     va_end(argList);
 63
 64     errno = savedErrno;
 65 }
 66
 67 void
 68 errExit(const char *format, ...)
 69 {
 70     va_list argList;
 71
 72     va_start(argList, format);
 73     outputError(TRUE, errno, TRUE, format, argList);
 74     va_end(argList);
 75
 76     terminate(TRUE);
 77 }
 78
 79 void
 80 err_exit(const char *format, ...)
 81 {
 82     va_list argList;
 83
 84     va_start(argList, format);
 85     outputError(TRUE, errno, FALSE, format, argList);
 86     va_end(argList);
 87
 88     terminate(FALSE);
 89 }
 90
 91 void
 92 errExitEN(int errnum, const char *format, ...)
 93 {
 94     va_list argList;
 95
 96     va_start(argList, format);
 97     outputError(TRUE, errnum, TRUE, format, argList);
 98     va_end(argList);
 99
100     terminate(TRUE);
101 }
102
103 void
104 fatal(const char *format, ...)
105 {
106     va_list argList;
107
108     va_start(argList, format);
109     outputError(FALSE, 0, TRUE, format, argList);
110     va_end(argList);
111
112     terminate(TRUE);
113 }
114
115 void
116 usageErr(const char *format, ...)
117 {
118     va_list argList;
119
120     fflush(stdout);           /* Flush any pending stdout */
121
122     fprintf(stderr, "Usage: ");
123     va_start(argList, format);
124     vfprintf(stderr, format, argList);
125     va_end(argList);
126
127     fflush(stderr);           /* In case stderr is not line-buffered */
128     exit(EXIT_FAILURE);
129 }
130
131 void
132 cmdLineErr(const char *format, ...)
133 {
134     va_list argList;
135
136     fflush(stdout);           /* Flush any pending stdout */
137
138     fprintf(stderr, "Command-line usage error: ");
139     va_start(argList, format);
140     vfprintf(stderr, format, argList);
141     va_end(argList);
142
143     fflush(stderr);           /* In case stderr is not line-buffered */
144     exit(EXIT_FAILURE);
145 }
