/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Lesser General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
\*************************************************************************/

/* Listing 3-6 */

 1 #include <stdio.h>
 2 #include <stdlib.h>
 3 #include <string.h>
 4 #include <limits.h>
 5 #include <errno.h>
 6 #include "get_num.h"
 7
 8 static void
 9 gnFail(const char *fname, const char *msg, const char *arg, const char *name)
10 {
11     fprintf(stderr, "%s error", fname);
12     if (name != NULL)
13         fprintf(stderr, " (in %s)", name);
14     fprintf(stderr, ": %s\n", msg);
15     if (arg != NULL && *arg != '\0')
16         fprintf(stderr, "        offending text: %s\n", arg);
17
18     exit(EXIT_FAILURE);
19 }
20
21 static long
22 getNum(const char *fname, const char *arg, int flags, const char *name)
23 {
24     long res;
25     char *endptr;
26     int base;
27
28     if (arg == NULL || *arg == '\0')
29         gnFail(fname, "null or empty string", arg, name);
30
31     base = (flags & GN_ANY_BASE) ? 0 : (flags & GN_BASE_8) ? 8 :
32                         (flags & GN_BASE_16) ? 16 : 10;
33
34     errno = 0;
35     res = strtol(arg, &endptr, base);
36     if (errno != 0)
37         gnFail(fname, "strtol() failed", arg, name);
38
39     if (*endptr != '\0')
40         gnFail(fname, "nonnumeric characters", arg, name);
41
42     if ((flags & GN_NONNEG) && res < 0)
43         gnFail(fname, "negative value not allowed", arg, name);
44
45     if ((flags & GN_GT_0) && res <= 0)
46         gnFail(fname, "value must be > 0", arg, name);
47
48     return res;
49 }
50
51 long
52 getLong(const char *arg, int flags, const char *name)
53 {
54     return getNum("getLong", arg, flags, name);
55 }
56
57 int
58 getInt(const char *arg, int flags, const char *name)
59 {
60     long res;
61
62     res = getNum("getInt", arg, flags, name);
63
64     if (res > INT_MAX || res < INT_MIN)
65         gnFail("getInt", "integer out of range", arg, name);
66
67     return (int) res;
68 }
