  1 /*************************************************************************\
  2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
  3 *                                                                         *
  4 * This program is free software. You may use, modify, and redistribute it *
  5 * under the terms of the GNU Lesser General Public License as published   *
  6 * by the Free Software Foundation, either version 3 or (at your option)   *
  7 * any later version. This program is distributed without any warranty.    *
  8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
  9 \*************************************************************************/
 10
 11 /* Listing 59-9 */
 12
 13   1 #define _BSD_SOURCE             /* To get NI_MAXHOST and NI_MAXSERV
 14   2                                    definitions from <netdb.h> */
 15   3 #include <sys/socket.h>
 16   4 #include <netinet/in.h>
 17   5 #include <arpa/inet.h>
 18   6 #include <netdb.h>
 19   7 #include "inet_sockets.h"       /* Declares functions defined here */
 20   8 #include "tlpi_hdr.h"
 21   9
 22  10 int
 23  11 inetConnect(const char *host, const char *service, int type)
 24  12 {
 25  13     struct addrinfo hints;
 26  14     struct addrinfo *result, *rp;
 27  15     int sfd, s;
 28  16
 29  17     memset(&hints, 0, sizeof(struct addrinfo));
 30  18     hints.ai_canonname = NULL;
 31  19     hints.ai_addr = NULL;
 32  20     hints.ai_next = NULL;
 33  21     hints.ai_family = AF_UNSPEC;        /* Allows IPv4 or IPv6 */
 34  22     hints.ai_socktype = type;
 35  23
 36  24     s = getaddrinfo(host, service, &hints, &result);
 37  25     if (s != 0) {
 38  26         errno = ENOSYS;
 39  27         return -1;
 40  28     }
 41  29
 42  30     /* Walk through returned list until we find an address structure
 43  31        that can be used to successfully connect a socket */
 44  32
 45  33     for (rp = result; rp != NULL; rp = rp->ai_next) {
 46  34         sfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
 47  35         if (sfd == -1)
 48  36             continue;                   /* On error, try next address */
 49  37
 50  38         if (connect(sfd, rp->ai_addr, rp->ai_addrlen) != -1)
 51  39             break;                      /* Success */
 52  40
 53  41         /* Connect failed: close this socket and try next address */
 54  42
 55  43         close(sfd);
 56  44     }
 57  45
 58  46     freeaddrinfo(result);
 59  47
 60  48     return (rp == NULL) ? -1 : sfd;
 61  49 }
 62  50
 63  51 static int              /* Public interfaces: inetBind() and inetListen() */
 64  52 inetPassiveSocket(const char *service, int type, socklen_t *addrlen,
 65  53                   Boolean doListen, int backlog)
 66  54 {
 67  55     struct addrinfo hints;
 68  56     struct addrinfo *result, *rp;
 69  57     int sfd, optval, s;
 70  58
 71  59     memset(&hints, 0, sizeof(struct addrinfo));
 72  60     hints.ai_canonname = NULL;
 73  61     hints.ai_addr = NULL;
 74  62     hints.ai_next = NULL;
 75  63     hints.ai_socktype = type;
 76  64     hints.ai_family = AF_UNSPEC;        /* Allows IPv4 or IPv6 */
 77  65     hints.ai_flags = AI_PASSIVE;        /* Use wildcard IP address */
 78  66
 79  67     s = getaddrinfo(NULL, service, &hints, &result);
 80  68     if (s != 0)
 81  69         return -1;
 82  70
 83  71     /* Walk through returned list until we find an address structure
 84  72        that can be used to successfully create and bind a socket */
 85  73
 86  74     optval = 1;
 87  75     for (rp = result; rp != NULL; rp = rp->ai_next) {
 88  76         sfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
 89  77         if (sfd == -1)
 90  78             continue;                   /* On error, try next address */
 91  79
 92  80         if (doListen) {
 93  81             if (setsockopt(sfd, SOL_SOCKET, SO_REUSEADDR, &optval,
 94  82                     sizeof(optval)) == -1) {
 95  83                 close(sfd);
 96  84                 freeaddrinfo(result);
 97  85                 return -1;
 98  86             }
 99  87         }
100  88
101  89         if (bind(sfd, rp->ai_addr, rp->ai_addrlen) == 0)
102  90             break;                      /* Success */
103  91
104  92         /* bind() failed: close this socket and try next address */
105  93
106  94         close(sfd);
107  95     }
108  96
109  97     if (rp != NULL && doListen) {
110  98         if (listen(sfd, backlog) == -1) {
111  99             freeaddrinfo(result);
112 100             return -1;
113 101         }
114 102     }
115 103
116 104     if (rp != NULL && addrlen != NULL)
117 105         *addrlen = rp->ai_addrlen;      /* Return address structure size */
118 106
119 107     freeaddrinfo(result);
120 108
121 109     return (rp == NULL) ? -1 : sfd;
122 110 }
123 111
124 112 int
125 113 inetListen(const char *service, int backlog, socklen_t *addrlen)
126 114 {
127 115     return inetPassiveSocket(service, SOCK_STREAM, addrlen, TRUE, backlog);
128 116 }
129 117
130 118 int
131 119 inetBind(const char *service, int type, socklen_t *addrlen)
132 120 {
133 121     return inetPassiveSocket(service, type, addrlen, FALSE, 0);
134 122 }
135 123
136 124 char *
137 125 inetAddressStr(const struct sockaddr *addr, socklen_t addrlen,
138 126                char *addrStr, int addrStrLen)
139 127 {
140 128     char host[NI_MAXHOST], service[NI_MAXSERV];
141 129
142 130     if (getnameinfo(addr, addrlen, host, NI_MAXHOST,
143 131                     service, NI_MAXSERV, NI_NUMERICSERV) == 0)
144 132         snprintf(addrStr, addrStrLen, "(%s, %s)", host, service);
145 133     else
146 134         snprintf(addrStr, addrStrLen, "(?UNKNOWN?)");
147 135
148 136     return addrStr;
149 137 }
