 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 64-1 */
12
13  1 #define _XOPEN_SOURCE 600
14  2 #include <stdlib.h>
15  3 #include <fcntl.h>
16  4 #include "pty_master_open.h"            /* Declares ptyMasterOpen() */
17  5 #include "tlpi_hdr.h"
18  6
19  7 int
20  8 ptyMasterOpen(char *slaveName, size_t snLen)
21  9 {
22 10     int masterFd, savedErrno;
23 11     char *p;
24 12
25 13     masterFd = posix_openpt(O_RDWR | O_NOCTTY);         /* Open pty master */
26 14     if (masterFd == -1)
27 15         return -1;
28 16
29 17     if (grantpt(masterFd) == -1) {              /* Grant access to slave pty */
30 18         savedErrno = errno;
31 19         close(masterFd);                        /* Might change 'errno' */
32 20         errno = savedErrno;
33 21         return -1;
34 22     }
35 23
36 24     if (unlockpt(masterFd) == -1) {             /* Unlock slave pty */
37 25         savedErrno = errno;
38 26         close(masterFd);                        /* Might change 'errno' */
39 27         errno = savedErrno;
40 28         return -1;
41 29     }
42 30
43 31     p = ptsname(masterFd);                      /* Get slave pty name */
44 32     if (p == NULL) {
45 33         savedErrno = errno;
46 34         close(masterFd);                        /* Might change 'errno' */
47 35         errno = savedErrno;
48 36         return -1;
49 37     }
50 38
51 39     if (strlen(p) < snLen) {
52 40         strncpy(slaveName, p, snLen);
53 41     } else {                    /* Return an error if buffer too small */
54 42         close(masterFd);
55 43         errno = EOVERFLOW;
56 44         return -1;
57 45     }
58 46
59 47     return masterFd;
60 48 }
