  1 /*************************************************************************\
  2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
  3 *                                                                         *
  4 * This program is free software. You may use, modify, and redistribute it *
  5 * under the terms of the GNU Lesser General Public License as published   *
  6 * by the Free Software Foundation, either version 3 or (at your option)   *
  7 * any later version. This program is distributed without any warranty.    *
  8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
  9 \*************************************************************************/
 10
 11 /* Solution for Exercise 59-3:b */
 12
 13  1 #include "unix_sockets.h"       /* Declares functions defined here */
 14  2 #include "tlpi_hdr.h"
 15  3
 16  4 int
 17  5 unixBuildAddress(const char *path, struct sockaddr_un *addr)
 18  6 {
 19  7     if (addr == NULL || path == NULL ||
 20  8             strlen(path) >= sizeof(addr->sun_path) - 1) {
 21  9         errno = EINVAL;
 22 10         return -1;
 23 11     }
 24 12
 25 13     memset(addr, 0, sizeof(struct sockaddr_un));
 26 14     addr->sun_family = AF_UNIX;
 27 15     if (strlen(path) < sizeof(addr->sun_path)) {
 28 16         strncpy(addr->sun_path, path, sizeof(addr->sun_path) - 1);
 29 17         return 0;
 30 18     } else {
 31 19         errno = ENAMETOOLONG;
 32 20         return -1;
 33 21     }
 34 22 }
 35 23
 36 24 int
 37 25 unixConnect(const char *path, int type)
 38 26 {
 39 27     int sd, savedErrno;
 40 28     struct sockaddr_un addr;
 41 29
 42 30     if (unixBuildAddress(path, &addr) == -1)
 43 31         return -1;
 44 32
 45 33     sd = socket(AF_UNIX, type, 0);
 46 34     if (sd == -1)
 47 35         return -1;
 48 36
 49 37     if (connect(sd, (struct sockaddr *) &addr,
 50 38                 sizeof(struct sockaddr_un)) == -1) {
 51 39         savedErrno = errno;
 52 40         close(sd);                      /* Might change 'errno' */
 53 41         errno = savedErrno;
 54 42         return -1;
 55 43     }
 56 44
 57 45     return sd;
 58 46 }
 59 47
 60 48 /* Create a UNIX domain socket and bind it to 'path'. If 'doListen'
 61 49    is true, then call listen() with specified 'backlog'.
 62 50    Return the socket descriptor on success, or -1 on error. */
 63 51
 64 52 static int              /* Public interfaces: unixBind() and unixListen() */
 65 53 unixPassiveSocket(const char *path, int type, Boolean doListen, int backlog)
 66 54 {
 67 55     int sd, savedErrno;
 68 56     struct sockaddr_un addr;
 69 57
 70 58     if (unixBuildAddress(path, &addr) == -1)
 71 59         return -1;
 72 60
 73 61     sd = socket(AF_UNIX, type, 0);
 74 62     if (sd == -1)
 75 63         return -1;
 76 64
 77 65     if (bind(sd, (struct sockaddr *) &addr, sizeof(struct sockaddr_un)) == -1) {
 78 66         savedErrno = errno;
 79 67         close(sd);                      /* Might change 'errno' */
 80 68         errno = savedErrno;
 81 69         return -1;
 82 70     }
 83 71
 84 72     if (doListen) {
 85 73         if (listen(sd, backlog) == -1) {
 86 74             savedErrno = errno;
 87 75             close(sd);                  /* Might change 'errno' */
 88 76             errno = savedErrno;
 89 77             return -1;
 90 78         }
 91 79     }
 92 80
 93 81     return sd;
 94 82 }
 95 83
 96 84 int
 97 85 unixListen(const char *path, int backlog)
 98 86 {
 99 87     return unixPassiveSocket(path, SOCK_STREAM, TRUE, backlog);
100 88 }
101 89
102 90 int
103 91 unixBind(const char *path, int type)
104 92 {
105 93     return unixPassiveSocket(path, type, FALSE, 0);
106 94 }
