/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 49-3 */

 1 #ifdef USE_MAP_ANON
 2 #define _BSD_SOURCE             /* Get MAP_ANONYMOUS definition */
 3 #endif
 4 #include <sys/wait.h>
 5 #include <sys/mman.h>
 6 #include <fcntl.h>
 7 #include "tlpi_hdr.h"
 8
 9 int
10 main(int argc, char *argv[])
11 {
12     int *addr;                  /* Pointer to shared memory region */
13
14 #ifdef USE_MAP_ANON             /* Use MAP_ANONYMOUS */
15     addr = mmap(NULL, sizeof(int), PROT_READ | PROT_WRITE,
16                 MAP_SHARED | MAP_ANONYMOUS, -1, 0);
17     if (addr == MAP_FAILED)
18         errExit("mmap");
19
20 #else                           /* Map /dev/zero */
21     int fd;
22
23     fd = open("/dev/zero", O_RDWR);
24     if (fd == -1)
25         errExit("open");
26
27     addr = mmap(NULL, sizeof(int), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
28     if (addr == MAP_FAILED)
29         errExit("mmap");
30
31     if (close(fd) == -1)        /* No longer needed */
32         errExit("close");
33 #endif
34
35     *addr = 1;                  /* Initialize integer in mapped region */
36
37     switch (fork()) {           /* Parent and child share mapping */
38     case -1:
39         errExit("fork");
40
41     case 0:                     /* Child: increment shared integer and exit */
42         printf("Child started, value = %d\n", *addr);
43         (*addr)++;
44         if (munmap(addr, sizeof(int)) == -1)
45             errExit("munmap");
46         exit(EXIT_SUCCESS);
47
48     default:                    /* Parent: wait for child to terminate */
49         if (wait(NULL) == -1)
50             errExit("wait");
51         printf("In parent, value = %d\n", *addr);
52         if (munmap(addr, sizeof(int)) == -1)
53             errExit("munmap");
54         exit(EXIT_SUCCESS);
55     }
56 }
