/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 34-5 */

 1 #define _GNU_SOURCE     /* Get declaration of strsignal() from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include <fcntl.h>
 5 #include "tlpi_hdr.h"
 6
 7 static int cmdNum;      /* Our position in pipeline */
 8
 9 static void             /* Handler for various signals */
10 handler(int sig)
11 {
12     /* UNSAFE: This handler uses non-async-signal-safe functions
13        (fprintf(), strsignal(); see Section 21.1.2) */
14
15     if (getpid() == getpgrp())          /* If process group leader */
16         fprintf(stderr, "Terminal FG process group: %ld\n",
17                 (long) tcgetpgrp(STDERR_FILENO));
18     fprintf(stderr, "Process %ld (%d) received signal %d (%s)\n",
19                 (long) getpid(), cmdNum, sig, strsignal(sig));
20
21     /* If we catch SIGTSTP, it won't actually stop us. Therefore we
22        raise SIGSTOP so we actually get stopped. */
23
24     if (sig == SIGTSTP)
25         raise(SIGSTOP);
26 }
27
28 int
29 main(int argc, char *argv[])
30 {
31     struct sigaction sa;
32
33     sigemptyset(&sa.sa_mask);
34     sa.sa_flags = SA_RESTART;
35     sa.sa_handler = handler;
36     if (sigaction(SIGINT, &sa, NULL) == -1)
37         errExit("sigaction");
38     if (sigaction(SIGTSTP, &sa, NULL) == -1)
39         errExit("sigaction");
40     if (sigaction(SIGCONT, &sa, NULL) == -1)
41         errExit("sigaction");
42
43     /* If stdin is a terminal, this is the first process in pipeline:
44        print a heading and initialize message to be sent down pipe */
45
46     if (isatty(STDIN_FILENO)) {
47         fprintf(stderr, "Terminal FG process group: %ld\n",
48                 (long) tcgetpgrp(STDIN_FILENO));
49         fprintf(stderr, "Command   PID  PPID  PGRP   SID\n");
50         cmdNum = 0;
51
52     } else {            /* Not first in pipeline, so read message from pipe */
53         if (read(STDIN_FILENO, &cmdNum, sizeof(cmdNum)) <= 0)
54             fatal("read got EOF or error");
55     }
56
57     cmdNum++;
58     fprintf(stderr, "%4d    %5ld %5ld %5ld %5ld\n", cmdNum,
59             (long) getpid(), (long) getppid(),
60             (long) getpgrp(), (long) getsid(0));
61
62     /* If not the last process, pass a message to the next process */
63
64     if (!isatty(STDOUT_FILENO))   /* If not tty, then should be pipe */
65         if (write(STDOUT_FILENO, &cmdNum, sizeof(cmdNum)) == -1)
66             errMsg("write");
67
68     for(;;)             /* Wait for signals */
69         pause();
70 }
