/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 44-1 */

 1 #include <ctype.h>
 2 #include "tlpi_hdr.h"
 3
 4 #define BUF_SIZE 100    /* Should be <= PIPE_BUF bytes */
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     char buf[BUF_SIZE];
10     int outbound[2];            /* Pipe to send data from parent to child */
11     int inbound[2];             /* Pipe to send data from child to parent */
12     int j;
13     ssize_t cnt;
14
15     if (pipe(outbound) == -1)
16         errExit("pipe");
17     if (pipe(inbound) == -1)
18         errExit("pipe");
19
20     switch (fork()) {
21     case -1:
22         errExit("fork");
23
24     case 0: /* Child */
25
26         /* Close unused pipe descriptors */
27
28         if (close(outbound[1]) == -1)
29             errExit("close");
30         if (close(inbound[0]) == -1)
31             errExit("close");
32
33         /* Read data from outbound pipe, convert to uppercase,
34            and send back to parent on inbound pipe */
35
36         while ((cnt = read(outbound[0], buf, BUF_SIZE)) > 0) {
37             for (j = 0; j < cnt; j++)
38                 buf[j] = toupper((unsigned char) buf[j]);
39             if (write(inbound[1], buf, cnt) == -1)
40                 errExit("write");
41         }
42
43         if (cnt == -1)
44             errExit("read");
45         _exit(EXIT_SUCCESS);
46
47     default:
48
49         /* Close unused pipe descriptors */
50
51         if (close(outbound[0]) == -1)
52             errExit("close");
53         if (close(inbound[1]) == -1)
54             errExit("close");
55
56         /* Read data from stdin, send to the child via the
57            outbound pipe, read the results back from the child
58            on the inbound pipe, and print them on stdout */
59
60         while ((cnt = read(STDIN_FILENO, buf, BUF_SIZE)) > 0) {
61             if (write(outbound[1], buf, cnt) == -1)
62                 errExit("write");
63
64             cnt = read(inbound[0], buf, BUF_SIZE);
65             if (cnt == -1)
66                 errExit("read");
67             if (cnt > 0)
68                 if (write(STDOUT_FILENO, buf, cnt) == -1)
69                     errExit("write");
70         }
71
72         if (cnt == -1)
73             errExit("read");
74
75         /* Exiting will close write end of outbound pipe, so that
76            child see EOF */
77
78         exit(EXIT_SUCCESS);
79     }
80 }
