/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 44-8 */

 1 #include "fifo_seqnum.h"
 2
 3 static char clientFifo[CLIENT_FIFO_NAME_LEN];
 4
 5 static void             /* Invoked on exit to delete client FIFO */
 6 removeFifo(void)
 7 {
 8     unlink(clientFifo);
 9 }
10
11 int
12 main(int argc, char *argv[])
13 {
14     int serverFd, clientFd;
15     struct request req;
16     struct response resp;
17
18     if (argc > 1 && strcmp(argv[1], "--help") == 0)
19         usageErr("%s [seq-len...]\n", argv[0]);
20
21     /* Create our FIFO (before sending request, to avoid a race) */
22
23     umask(0);                   /* So we get the permissions we want */
24     snprintf(clientFifo, CLIENT_FIFO_NAME_LEN, CLIENT_FIFO_TEMPLATE,
25             (long) getpid());
26     if (mkfifo(clientFifo, S_IRUSR | S_IWUSR | S_IWGRP) == -1
27                 && errno != EEXIST)
28         errExit("mkfifo %s", clientFifo);
29
30     if (atexit(removeFifo) != 0)
31         errExit("atexit");
32
33     /* Construct request message, open server FIFO, and send message */
34
35     req.pid = getpid();
36     req.seqLen = (argc > 1) ? getInt(argv[1], GN_GT_0, "seq-len") : 1;
37
38     serverFd = open(SERVER_FIFO, O_WRONLY);
39     if (serverFd == -1)
40         errExit("open %s", SERVER_FIFO);
41
42     if (write(serverFd, &req, sizeof(struct request)) !=
43             sizeof(struct request))
44         fatal("Can't write to server");
45
46     /* Open our FIFO, read and display response */
47
48     clientFd = open(clientFifo, O_RDONLY);
49     if (clientFd == -1)
50         errExit("open %s", clientFifo);
51
52     if (read(clientFd, &resp, sizeof(struct response))
53             != sizeof(struct response))
54         fatal("Can't read response from server");
55
56     printf("%d\n", resp.seqNum);
57     exit(EXIT_SUCCESS);
58 }
