/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 44-4 */

 1 #include <sys/wait.h>
 2 #include "tlpi_hdr.h"
 3
 4 int
 5 main(int argc, char *argv[])
 6 {
 7     int pfd[2];                                     /* Pipe file descriptors */
 8
 9     if (pipe(pfd) == -1)                            /* Create pipe */
10         errExit("pipe");
11
12     switch (fork()) {
13     case -1:
14         errExit("fork");
15
16     case 0:             /* First child: exec 'ls' to write to pipe */
17         if (close(pfd[0]) == -1)                    /* Read end is unused */
18             errExit("close 1");
19
20         /* Duplicate stdout on write end of pipe; close duplicated descriptor */
21
22         if (pfd[1] != STDOUT_FILENO) {              /* Defensive check */
23             if (dup2(pfd[1], STDOUT_FILENO) == -1)
24                 errExit("dup2 1");
25             if (close(pfd[1]) == -1)
26                 errExit("close 2");
27         }
28
29         execlp("ls", "ls", (char *) NULL);          /* Writes to pipe */
30         errExit("execlp ls");
31
32     default:            /* Parent falls through to create next child */
33         break;
34     }
35
36     switch (fork()) {
37     case -1:
38         errExit("fork");
39
40     case 0:             /* Second child: exec 'wc' to read from pipe */
41         if (close(pfd[1]) == -1)                    /* Write end is unused */
42             errExit("close 3");
43
44         /* Duplicate stdin on read end of pipe; close duplicated descriptor */
45
46         if (pfd[0] != STDIN_FILENO) {               /* Defensive check */
47             if (dup2(pfd[0], STDIN_FILENO) == -1)
48                 errExit("dup2 2");
49             if (close(pfd[0]) == -1)
50                 errExit("close 4");
51         }
52
53         execlp("wc", "wc", "-l", (char *) NULL);
54         errExit("execlp wc");
55
56     default: /* Parent falls through */
57         break;
58     }
59
60     /* Parent closes unused file descriptors for pipe, and waits for children */
61
62     if (close(pfd[0]) == -1)
63         errExit("close 5");
64     if (close(pfd[1]) == -1)
65         errExit("close 6");
66     if (wait(NULL) == -1)
67         errExit("wait 1");
68     if (wait(NULL) == -1)
69         errExit("wait 2");
70
71     exit(EXIT_SUCCESS);
72 }
