/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 44-5 */

 1 #include <ctype.h>
 2 #include <limits.h>
 3 #include "print_wait_status.h"          /* For printWaitStatus() */
 4 #include "tlpi_hdr.h"
 5
 6 #define POPEN_FMT "/bin/ls -d %s 2> /dev/null"
 7 #define PAT_SIZE 50
 8 #define PCMD_BUF_SIZE (sizeof(POPEN_FMT) + PAT_SIZE)
 9
10 int
11 main(int argc, char *argv[])
12 {
13     char pat[PAT_SIZE];                 /* Pattern for globbing */
14     char popenCmd[PCMD_BUF_SIZE];
15     FILE *fp;                           /* File stream returned by popen() */
16     Boolean badPattern;                 /* Invalid characters in 'pat'? */
17     int len, status, fileCnt, j;
18     char pathname[PATH_MAX];
19
20     for (;;) {                  /* Read pattern, display results of globbing */
21         printf("pattern: ");
22         fflush(stdout);
23         if (fgets(pat, PAT_SIZE, stdin) == NULL)
24             break;                      /* EOF */
25         len = strlen(pat);
26         if (len <= 1)                   /* Empty line */
27             continue;
28
29         if (pat[len - 1] == '\n')       /* Strip trailing newline */
30             pat[len - 1] = '\0';
31
32         /* Ensure that the pattern contains only valid characters,
33            i.e., letters, digits, underscore, dot, and the shell
34            globbing characters. (Our definition of valid is more
35            restrictive than the shell, which permits other characters
36            to be included in a filename if they are quoted.) */
37
38         for (j = 0, badPattern = FALSE; j < len && !badPattern; j++)
39             if (!isalnum((unsigned char) pat[j]) &&
40                     strchr("_*?[^-].", pat[j]) == NULL)
41                 badPattern = TRUE;
42
43         if (badPattern) {
44             printf("Bad pattern character: %c\n", pat[j - 1]);
45             continue;
46         }
47
48         /* Build and execute command to glob 'pat' */
49
50         snprintf(popenCmd, PCMD_BUF_SIZE, POPEN_FMT, pat);
51
52         fp = popen(popenCmd, "r");
53         if (fp == NULL) {
54             printf("popen() failed\n");
55             continue;
56         }
57
58         /* Read resulting list of pathnames until EOF */
59
60         fileCnt = 0;
61         while (fgets(pathname, PATH_MAX, fp) != NULL) {
62             printf("%s", pathname);
63             fileCnt++;
64         }
65
66         /* Close pipe, fetch and display termination status */
67
68         status = pclose(fp);
69         printf("    %d matching file%s\n", fileCnt, (fileCnt != 1) ? "s" : "");
70         printf("    pclose() status = %#x\n", (unsigned int) status);
71         if (status != -1)
72             printWaitStatus("\t", status);
73     }
74
75     exit(EXIT_SUCCESS);
76 }
