/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 44-2 */

 1 #include <sys/wait.h>
 2 #include "tlpi_hdr.h"
 3
 4 #define BUF_SIZE 10
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     int pfd[2];                             /* Pipe file descriptors */
10     char buf[BUF_SIZE];
11     ssize_t numRead;
12
13     if (argc != 2 || strcmp(argv[1], "--help") == 0)
14         usageErr("%s string\n", argv[0]);
15
16     if (pipe(pfd) == -1)                    /* Create the pipe */
17         errExit("pipe");
18
19     switch (fork()) {
20     case -1:
21         errExit("fork");
22
23     case 0:             /* Child  - reads from pipe */
24         if (close(pfd[1]) == -1)            /* Write end is unused */
25             errExit("close - child");
26
27         for (;;) {              /* Read data from pipe, echo on stdout */
28             numRead = read(pfd[0], buf, BUF_SIZE);
29             if (numRead == -1)
30                 errExit("read");
31             if (numRead == 0)
32                 break;                      /* End-of-file */
33             if (write(STDOUT_FILENO, buf, numRead) != numRead)
34                 fatal("child - partial/failed write");
35         }
36
37         write(STDOUT_FILENO, "\n", 1);
38         if (close(pfd[0]) == -1)
39             errExit("close");
40         _exit(EXIT_SUCCESS);
41
42     default:            /* Parent - writes to pipe */
43         if (close(pfd[0]) == -1)            /* Read end is unused */
44             errExit("close - parent");
45
46         if (write(pfd[1], argv[1], strlen(argv[1])) != strlen(argv[1]))
47             fatal("parent - partial/failed write");
48
49         if (close(pfd[1]) == -1)            /* Child will see EOF */
50             errExit("close");
51         wait(NULL);                         /* Wait for child to finish */
52         exit(EXIT_SUCCESS);
53     }
54 }
