/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 52-6 */

 1 #define _POSIX_C_SOURCE 199309
 2 #include <signal.h>
 3 #include <mqueue.h>
 4 #include <fcntl.h>              /* For definition of O_NONBLOCK */
 5 #include "tlpi_hdr.h"
 6
 7 #define NOTIFY_SIG SIGRTMIN     /* Signal used for message notifications */
 8
 9 int
10 main(int argc, char *argv[])
11 {
12     struct sigevent sev;
13     mqd_t mqd;
14     struct mq_attr attr;
15     void *buffer;
16     ssize_t numRead;
17     sigset_t blockMask;
18     siginfo_t si;
19
20     if (argc != 2 || strcmp(argv[1], "--help") == 0)
21         usageErr("%s mq-name\n", argv[0]);
22
23     mqd = mq_open(argv[1], O_RDONLY | O_NONBLOCK);
24     if (mqd == (mqd_t) -1)
25         errExit("mq_open");
26
27     /* Determine mq_msgsize for message queue, and allocate an input buffer
28        of that size */
29
30     if (mq_getattr(mqd, &attr) == -1)
31         errExit("mq_getattr");
32     buffer = malloc(attr.mq_msgsize);
33     if (buffer == NULL)
34         errExit("malloc");
35
36     /* Block the signal that we'll accept using sigwaitinfo() */
37
38     sigemptyset(&blockMask);
39     sigaddset(&blockMask, NOTIFY_SIG);
40     if (sigprocmask(SIG_BLOCK, &blockMask, NULL) == -1)
41         errExit("sigprocmask");
42
43     /* Set up message notification using the signal NOTIFY_SIG */
44
45     sev.sigev_notify = SIGEV_SIGNAL;
46     sev.sigev_signo = NOTIFY_SIG;
47     sev.sigev_value.sival_ptr = &mqd;
48                 /* This allows us to obtain a pointer to 'mqd' in the
49                    siginfo_t structure returned by sigwaitinfo() */
50
51     if (mq_notify(mqd, &sev) == -1)
52         errExit("mq_notify");
53
54     for (;;) {
55
56         /* Wait for a signal; when it is received, display associated
57            information */
58
59         if (sigwaitinfo(&blockMask, &si) == -1)errExit("sigwaitinfo");
60
61         printf("Accepted signal:\n");
62         printf("        si_signo   = %d\n", si.si_signo);
63         printf("        si_pid     = %ld\n", (long) si.si_pid);
64         printf("        si_uid     = %ld\n", (long) si.si_uid);
65         printf("        si_code    = %d (%s)\n", si.si_code,
66                 (si.si_code == SI_MESGQ) ? "SI_MESGQ" : "???");
67         printf("        *sival_ptr = %p\n\n", si.si_value.sival_ptr);
68
69         /* Reestablish message notification */
70
71         if (mq_notify(mqd, &sev) == -1)
72             errExit("mq_notify");
73
74         /* Although only one signal might have been queued (if NOTIFY_SIG
75            is a standard signal) we might have received multiple messages,
76            so use nonblocking mq_receive() calls inside a loop to read
77            as many messages as possible. */
78
79         while ((numRead = mq_receive(mqd, buffer, attr.mq_msgsize, NULL)) >= 0)
80             printf("Read %ld bytes\n", (long) numRead);
81
82         if (errno != EAGAIN)            /* Unexpected error */
83             errExit("mq_receive");
84     }
85 }
