/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 52-7 */

 1 #include <pthread.h>
 2 #include <mqueue.h>
 3 #include <fcntl.h>              /* For definition of O_NONBLOCK */
 4 #include "tlpi_hdr.h"
 5
 6 static void notifySetup(mqd_t *mqdp);
 7
 8 static void                     /* Thread notification function */
 9 threadFunc(union sigval sv)
10 {
11     ssize_t numRead;
12     mqd_t *mqdp;
13     void *buffer;
14     struct mq_attr attr;
15
16     mqdp = sv.sival_ptr;
17
18     /* Determine mq_msgsize for message queue, and allocate an input buffer
19        of that size */
20
21     if (mq_getattr(*mqdp, &attr) == -1)
22         errExit("mq_getattr");
23
24     buffer = malloc(attr.mq_msgsize);
25     if (buffer == NULL)
26         errExit("malloc");
27
28     notifySetup(mqdp);
29
30     while ((numRead = mq_receive(*mqdp, buffer, attr.mq_msgsize, NULL)) >= 0)
31         printf("Read %ld bytes\n", (long) numRead);
32
33     if (errno != EAGAIN)                        /* Unexpected error */
34         errExit("mq_receive");
35
36     free(buffer);
37 }
38
39 static void
40 notifySetup(mqd_t *mqdp)
41 {
42     struct sigevent sev;
43
44     sev.sigev_notify = SIGEV_THREAD;            /* Notify via thread */
45     sev.sigev_notify_function = threadFunc;
46     sev.sigev_notify_attributes = NULL;
47             /* Could be pointer to pthread_attr_t structure */
48     sev.sigev_value.sival_ptr = mqdp;           /* Argument to threadFunc() */
49
50     if (mq_notify(*mqdp, &sev) == -1)
51         errExit("mq_notify");
52 }
53
54 int
55 main(int argc, char *argv[])
56 {
57     mqd_t mqd;
58
59     if (argc != 2 || strcmp(argv[1], "--help") == 0)
60         usageErr("%s mq-name\n", argv[0]);
61
62     mqd = mq_open(argv[1], O_RDONLY | O_NONBLOCK);
63     if (mqd == (mqd_t) -1)
64         errExit("mq_open");
65
66     notifySetup(&mqd);
67     pause();                    /* Wait for notifications via thread function */
68 }
