/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 52-2 */

 1 #include <mqueue.h>
 2 #include <sys/stat.h>
 3 #include <fcntl.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void
 7 usageError(const char *progName)
 8 {
 9     fprintf(stderr, "Usage: %s [-cx] [-m maxmsg] [-s msgsize] mq-name "
10             "[octal-perms]\n", progName);
11     fprintf(stderr, "    -c          Create queue (O_CREAT)\n");
12     fprintf(stderr, "    -m maxmsg   Set maximum # of messages\n");
13     fprintf(stderr, "    -s msgsize  Set maximum message size\n");
14     fprintf(stderr, "    -x          Create exclusively (O_EXCL)\n");
15     exit(EXIT_FAILURE);
16 }
17
18 int
19 main(int argc, char *argv[])
20 {
21     int flags, opt;
22     mode_t perms;
23     mqd_t mqd;
24     struct mq_attr attr, *attrp;
25
26     attrp = NULL;
27     attr.mq_maxmsg = 50;
28     attr.mq_msgsize = 2048;
29     flags = O_RDWR;
30
31     /* Parse command-line options */
32
33     while ((opt = getopt(argc, argv, "cm:s:x")) != -1) {
34         switch (opt) {
35         case 'c':
36             flags |= O_CREAT;
37             break;
38
39         case 'm':
40             attr.mq_maxmsg = atoi(optarg);
41             attrp = &attr;
42             break;
43
44         case 's':
45             attr.mq_msgsize = atoi(optarg);
46             attrp = &attr;
47             break;
48
49         case 'x':
50             flags |= O_EXCL;
51             break;
52
53         default:
54             usageError(argv[0]);
55         }
56     }
57
58     if (optind >= argc)
59         usageError(argv[0]);
60
61     perms = (argc <= optind + 1) ? (S_IRUSR | S_IWUSR) :
62                 getInt(argv[optind + 1], GN_BASE_8, "octal-perms");
63
64     mqd = mq_open(argv[optind], flags, perms, attrp);
65     if (mqd == (mqd_t) -1)
66         errExit("mq_open");
67
68     exit(EXIT_SUCCESS);
69 }
