/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 52-5 */

 1 #include <mqueue.h>
 2 #include <fcntl.h>              /* For definition of O_NONBLOCK */
 3 #include "tlpi_hdr.h"
 4
 5 static void
 6 usageError(const char *progName)
 7 {
 8     fprintf(stderr, "Usage: %s [-n] mq-name\n", progName);
 9     fprintf(stderr, "    -n           Use O_NONBLOCK flag\n");
10     exit(EXIT_FAILURE);
11 }
12
13 int
14 main(int argc, char *argv[])
15 {
16     int flags, opt;
17     mqd_t mqd;
18     unsigned int prio;
19     void *buffer;
20     struct mq_attr attr;
21     ssize_t numRead;
22
23     flags = O_RDONLY;
24     while ((opt = getopt(argc, argv, "n")) != -1) {
25         switch (opt) {
26         case 'n':   flags |= O_NONBLOCK;        break;
27         default:    usageError(argv[0]);
28         }
29     }
30
31     if (optind >= argc)
32         usageError(argv[0]);
33
34     mqd = mq_open(argv[optind], flags);
35     if (mqd == (mqd_t) -1)
36         errExit("mq_open");
37
38     if (mq_getattr(mqd, &attr) == -1)
39         errExit("mq_getattr");
40
41     buffer = malloc(attr.mq_msgsize);
42     if (buffer == NULL)
43         errExit("malloc");
44
45     numRead = mq_receive(mqd, buffer, attr.mq_msgsize, &prio);
46     if (numRead == -1)
47         errExit("mq_receive");
48
49     printf("Read %ld bytes; priority = %u\n", (long) numRead, prio);
50     if (write(STDOUT_FILENO, buffer, numRead) == -1)
51         errExit("write");
52     write(STDOUT_FILENO, "\n", 1);
53
54     exit(EXIT_SUCCESS);
55 }
