/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 6-3 */

 1 #include <stdlib.h>
 2 #include <stdio.h>
 3 #include <string.h>
 4 #include <errno.h>
 5
 6 int
 7 unsetenv(const char *name)
 8 {
 9     extern char **environ;
10     char **ep, **sp;
11     size_t len;
12
13     if (name == NULL || name[0] == '\0' || strchr(name, '=') != NULL) {
14         errno = EINVAL;
15         return -1;
16     }
17
18     len = strlen(name);
19
20     for (ep = environ; *ep != NULL; )
21         if (strncmp(*ep, name, len) == 0 && (*ep)[len] == '=') {
22
23             /* Remove found entry by shifting all successive entries
24                back one element */
25
26             for (sp = ep; *sp != NULL; sp++)
27                 *sp = *(sp + 1);
28
29             /* Continue around the loop to further instances of 'name' */
30
31         } else {
32             ep++;
33         }
34
35     return 0;
36 }
37
38 int
39 setenv(const char *name, const char *value, int overwrite)
40 {
41     char *es;
42
43     if (name == NULL || name[0] == '\0' || strchr(name, '=') != NULL ||
44             value == NULL) {
45         errno = EINVAL;
46         return -1;
47     }
48
49     if (getenv(name) != NULL && overwrite == 0)
50         return 0;
51
52     unsetenv(name);             /* Remove all occurrences */
53
54     es = malloc(strlen(name) + strlen(value) + 2);
55                                 /* +2 for '=' and null terminator */
56     if (es == NULL)
57         return -1;
58
59     strcpy(es, name);
60     strcat(es, "=");
61     strcat(es, value);
62
63     return (putenv(es) != 0) ? -1 : 0;
64 }
65
66 #ifdef TEST_IT
67
68 int
69 main()
70 {
71     if (putenv("TT=xxxxx") != 0)
72         perror("putenv");
73
74     system("echo '***** Environment before unsetenv(TT)'; "
75             "printenv | grep ^TT");
76     system("echo 'Total lines from printenv:' `printenv | wc -l`");
77
78     unsetenv("TT");
79
80     system("echo '***** Environment after unsetenv(TT)'; "
81             "printenv | grep ^TT");
82     system("echo 'Total lines from printenv:' `printenv | wc -l`");
83
84     setenv("xyz", "one", 1);
85     setenv("xyz", "two", 0);
86     setenv("xyz2", "222", 0);
87
88     system("echo '***** Environment after setenv() calls'; "
89             "printenv | grep ^x");
90     system("echo 'Total lines from printenv:' `printenv | wc -l`");
91
92     exit(EXIT_SUCCESS);
93 }
94
95 #endif
