/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 9-1 */

 1 #define _GNU_SOURCE
 2 #include <unistd.h>
 3 #include <sys/fsuid.h>
 4 #include <limits.h>
 5 #include "ugid_functions.h"   /* userNameFromId() & groupNameFromId() */
 6 #include "tlpi_hdr.h"
 7
 8 #define SG_SIZE (NGROUPS_MAX + 1)
 9
10 int
11 main(int argc, char *argv[])
12 {
13     uid_t ruid, euid, suid, fsuid;
14     gid_t rgid, egid, sgid, fsgid;
15     gid_t suppGroups[SG_SIZE];
16     int numGroups, j;
17     char *p;
18
19     if (getresuid(&ruid, &euid, &suid) == -1)
20         errExit("getresuid");
21     if (getresgid(&rgid, &egid, &sgid) == -1)
22         errExit("getresgid");
23
24     /* Attempts to change the file-system IDs are always ignored
25        for unprivileged processes, but even so, the following
26        calls return the current file-system IDs */
27
28     fsuid = setfsuid(0);
29     fsgid = setfsgid(0);
30
31     printf("UID: ");
32     p = userNameFromId(ruid);
33     printf("real=%s (%ld); ", (p == NULL) ? "???" : p, (long) ruid);
34     p = userNameFromId(euid);
35     printf("eff=%s (%ld); ", (p == NULL) ? "???" : p, (long) euid);
36     p = userNameFromId(suid);
37     printf("saved=%s (%ld); ", (p == NULL) ? "???" : p, (long) suid);
38     p = userNameFromId(fsuid);
39     printf("fs=%s (%ld); ", (p == NULL) ? "???" : p, (long) fsuid);
40     printf("\n");
41
42     printf("GID: ");
43     p = groupNameFromId(rgid);
44     printf("real=%s (%ld); ", (p == NULL) ? "???" : p, (long) rgid);
45     p = groupNameFromId(egid);
46     printf("eff=%s (%ld); ", (p == NULL) ? "???" : p, (long) egid);
47     p = groupNameFromId(sgid);
48     printf("saved=%s (%ld); ", (p == NULL) ? "???" : p, (long) sgid);
49     p = groupNameFromId(fsgid);
50     printf("fs=%s (%ld); ", (p == NULL) ? "???" : p, (long) fsgid);
51     printf("\n");
52
53     numGroups = getgroups(SG_SIZE, suppGroups);
54     if (numGroups == -1)
55         errExit("getgroups");
56
57     printf("Supplementary groups (%d): ", numGroups);
58     for (j = 0; j < numGroups; j++) {
59         p = groupNameFromId(suppGroups[j]);
60         printf("%s (%ld) ", (p == NULL) ? "???" : p, (long) suppGroups[j]);
61     }
62     printf("\n");
63
64     exit(EXIT_SUCCESS);
65 }
