/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 28 */

  1 #include <fcntl.h>
  2 #include <time.h>
  3 #include <sys/stat.h>
  4 #include <sys/acct.h>
  5 #include <limits.h>
  6 #include "ugid_functions.h"             /* Declaration of userNameFromId() */
  7 #include "tlpi_hdr.h"
  8
  9 #define TIME_BUF_SIZE 100
 10
 11 #define GLIBC_DEFINES_ACCT_V3           /* This is true since glibc 2.6 */
 12
 13 #ifndef GLIBC_DEFINES_ACCT_V3
 14 struct acct_v3
 15 {
 16     char        ac_flag;
 17     char        ac_version;
 18     u_int16_t   ac_tty;
 19     u_int32_t   ac_exitcode;
 20     u_int32_t   ac_uid;
 21     u_int32_t   ac_gid;
 22     u_int32_t   ac_pid;
 23     u_int32_t   ac_ppid;
 24     u_int32_t   ac_btime;
 25     float       ac_etime;
 26     comp_t      ac_utime;
 27     comp_t      ac_stime;
 28     comp_t      ac_mem;
 29     comp_t      ac_io;
 30     comp_t      ac_rw;
 31     comp_t      ac_minflt;
 32     comp_t      ac_majflt;
 33     comp_t      ac_swaps;
 34     char        ac_comm[ACCT_COMM];
 35 };
 36
 37 #endif
 38
 39 static long long                /* Convert comp_t value into long long */
 40 comptToLL(comp_t ct)
 41 {
 42     const int EXP_SIZE = 3;             /* 3-bit, base-8 exponent */
 43     const int MANTISSA_SIZE = 13;       /* Followed by 13-bit mantissa */
 44     const int MANTISSA_MASK = (1 << MANTISSA_SIZE) - 1;
 45     long long mantissa, exp;
 46
 47     mantissa = ct & MANTISSA_MASK;
 48     exp = (ct >> MANTISSA_SIZE) & ((1 << EXP_SIZE) - 1);
 49     return mantissa << (exp * 3);       /* Power of 8 = left shift 3 bits */
 50 }
 51
 52 int
 53 main(int argc, char *argv[])
 54 {
 55     int acctFile;
 56     struct acct_v3 ac;
 57     ssize_t numRead;
 58     char *s;
 59     char timeBuf[TIME_BUF_SIZE];
 60     struct tm *loc;
 61     time_t t;
 62
 63     if (argc != 2 || strcmp(argv[1], "--help") == 0)
 64         usageErr("%s file\n", argv[0]);
 65
 66     acctFile = open(argv[1], O_RDONLY);
 67     if (acctFile == -1)
 68         errExit("open");
 69
 70     printf("ver. command    flags   term.   PID   PPID  user     group"
 71            "      start date+time     CPU   elapsed\n");
 72     printf("                       status                             "
 73            "                          time    time\n");
 74
 75     while ((numRead = read(acctFile, &ac, sizeof(struct acct_v3))) > 0) {
 76         if (numRead != sizeof(struct acct_v3))
 77             fatal("partial read");
 78
 79         printf("%1d    ", (int) ac.ac_version);
 80         printf("%-8.8s   ", ac.ac_comm);
 81
 82         printf("%c", (ac.ac_flag & AFORK) ? 'F' : '-') ;
 83         printf("%c", (ac.ac_flag & ASU)   ? 'S' : '-') ;
 84         printf("%c", (ac.ac_flag & AXSIG) ? 'X' : '-') ;
 85         printf("%c", (ac.ac_flag & ACORE) ? 'C' : '-') ;
 86
 87         printf(" %#6lx   ", (unsigned long) ac.ac_exitcode);
 88
 89         printf(" %5ld %5ld  ", (long) ac.ac_pid, (long) ac.ac_ppid);
 90
 91         s = userNameFromId(ac.ac_uid);
 92         printf("%-8.8s ", (s == NULL) ? "???" : s);
 93
 94         s = groupNameFromId(ac.ac_gid);
 95         printf("%-8.8s ", (s == NULL) ? "???" : s);
 96
 97         t = ac.ac_btime;
 98         loc = localtime(&t);
 99         if (loc == NULL) {
100             printf("???Unknown time???  ");
101         } else {
102             strftime(timeBuf, TIME_BUF_SIZE, "%Y-%m-%d %T ", loc);
103             printf("%s ", timeBuf);
104         }
105
106         printf("%5.2f %7.2f ", (double) (comptToLL(ac.ac_utime) +
107                     comptToLL(ac.ac_stime)) / sysconf(_SC_CLK_TCK),
108                 ac.ac_etime / sysconf(_SC_CLK_TCK));
109         printf("\n");
110     }
111
112     if (numRead == -1)
113         errExit("read");
114
115     exit(EXIT_SUCCESS);
116 }
