/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 28-2 */

 1 #include <fcntl.h>
 2 #include <time.h>
 3 #include <sys/stat.h>
 4 #include <sys/acct.h>
 5 #include <limits.h>
 6 #include "ugid_functions.h"             /* Declaration of userNameFromId() */
 7 #include "tlpi_hdr.h"
 8
 9 #define TIME_BUF_SIZE 100
10
11 static long long                /* Convert comp_t value into long long */
12 comptToLL(comp_t ct)
13 {
14     const int EXP_SIZE = 3;             /* 3-bit, base-8 exponent */
15     const int MANTISSA_SIZE = 13;       /* Followed by 13-bit mantissa */
16     const int MANTISSA_MASK = (1 << MANTISSA_SIZE) - 1;
17     long long mantissa, exp;
18
19     mantissa = ct & MANTISSA_MASK;
20     exp = (ct >> MANTISSA_SIZE) & ((1 << EXP_SIZE) - 1);
21     return mantissa << (exp * 3);       /* Power of 8 = left shift 3 bits */
22 }
23
24 int
25 main(int argc, char *argv[])
26 {
27     int acctFile;
28     struct acct ac;
29     ssize_t numRead;
30     char *s;
31     char timeBuf[TIME_BUF_SIZE];
32     struct tm *loc;
33     time_t t;
34
35     if (argc != 2 || strcmp(argv[1], "--help") == 0)
36         usageErr("%s file\n", argv[0]);
37
38     acctFile = open(argv[1], O_RDONLY);
39     if (acctFile == -1)
40         errExit("open");
41
42     printf("command  flags   term.  user     "
43             "start time            CPU   elapsed\n");
44     printf("                status           "
45             "                      time    time\n");
46
47     while ((numRead = read(acctFile, &ac, sizeof(struct acct))) > 0) {
48         if (numRead != sizeof(struct acct))
49             fatal("partial read");
50
51         printf("%-8.8s  ", ac.ac_comm);
52
53         printf("%c", (ac.ac_flag & AFORK) ? 'F' : '-') ;
54         printf("%c", (ac.ac_flag & ASU)   ? 'S' : '-') ;
55         printf("%c", (ac.ac_flag & AXSIG) ? 'X' : '-') ;
56         printf("%c", (ac.ac_flag & ACORE) ? 'C' : '-') ;
57
58 #ifdef __linux__
59         printf(" %#6lx   ", (unsigned long) ac.ac_exitcode);
60 #else   /* Many other implementations provide ac_stat instead */
61         printf(" %#6lx   ", (unsigned long) ac.ac_stat);
62 #endif
63
64         s = userNameFromId(ac.ac_uid);
65         printf("%-8.8s ", (s == NULL) ? "???" : s);
66
67         t = ac.ac_btime;
68         loc = localtime(&t);
69         if (loc == NULL) {
70             printf("???Unknown time???  ");
71         } else {
72             strftime(timeBuf, TIME_BUF_SIZE, "%Y-%m-%d %T ", loc);
73             printf("%s ", timeBuf);
74         }
75
76         printf("%5.2f %7.2f ", (double) (comptToLL(ac.ac_utime) +
77                     comptToLL(ac.ac_stime)) / sysconf(_SC_CLK_TCK),
78                 (double) comptToLL(ac.ac_etime) / sysconf(_SC_CLK_TCK));
79         printf("\n");
80     }
81
82     if (numRead == -1)
83         errExit("read");
84
85     exit(EXIT_SUCCESS);
86 }
