/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 26-3 */

 1 #include <sys/wait.h>
 2 #include "print_wait_status.h"          /* Declares printWaitStatus() */
 3 #include "tlpi_hdr.h"
 4
 5 int
 6 main(int argc, char *argv[])
 7 {
 8     int status;
 9     pid_t childPid;
10
11     if (argc > 1 && strcmp(argv[1], "--help") == 0)
12         usageErr("%s [exit-status]\n", argv[0]);
13
14     switch (fork()) {
15     case -1: errExit("fork");
16
17     case 0:             /* Child: either exits immediately with given
18                            status or loops waiting for signals */
19         printf("Child started with PID = %ld\n", (long) getpid());
20         if (argc > 1)                   /* Status supplied on command line? */
21             exit(getInt(argv[1], 0, "exit-status"));
22         else                            /* Otherwise, wait for signals */
23             for (;;)
24                 pause();
25         exit(EXIT_FAILURE);             /* Not reached, but good practice */
26
27     default:            /* Parent: repeatedly wait on child until it
28                            either exits or is terminated by a signal */
29         for (;;) {
30             childPid = waitpid(-1, &status, WUNTRACED
31 #ifdef WCONTINUED       /* Not present on older versions of Linux */
32                                                 | WCONTINUED
33 #endif
34                     );
35             if (childPid == -1)
36                 errExit("waitpid");
37
38             /* Print status in hex, and as separate decimal bytes */
39
40             printf("waitpid() returned: PID=%ld; status=0x%04x (%d,%d)\n",
41                     (long) childPid,
42                     (unsigned int) status, status >> 8, status & 0xff);
43             printWaitStatus(NULL, status);
44
45             if (WIFEXITED(status) || WIFSIGNALED(status))
46                 exit(EXIT_SUCCESS);
47         }
48     }
49 }
