/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 24-2 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include <sys/wait.h>
 4 #include "tlpi_hdr.h"
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     int fd, flags;
10     char template[] = "/tmp/testXXXXXX";
11
12     setbuf(stdout, NULL);                   /* Disable buffering of stdout */
13
14     fd = mkstemp(template);
15     if (fd == -1)
16         errExit("mkstemp");
17
18     printf("File offset before fork(): %lld\n",
19             (long long) lseek(fd, 0, SEEK_CUR));
20
21     flags = fcntl(fd, F_GETFL);
22     if (flags == -1)
23         errExit("fcntl - F_GETFL");
24     printf("O_APPEND flag before fork() is: %s\n",
25             (flags & O_APPEND) ? "on" : "off");
26
27     switch (fork()) {
28     case -1:
29         errExit("fork");
30
31     case 0:     /* Child: change file offset and status flags */
32         if (lseek(fd, 1000, SEEK_SET) == -1)
33             errExit("lseek");
34
35         flags = fcntl(fd, F_GETFL);         /* Fetch current flags */
36         if (flags == -1)
37             errExit("fcntl - F_GETFL");
38         flags |= O_APPEND;                  /* Turn O_APPEND on */
39         if (fcntl(fd, F_SETFL, flags) == -1)
40             errExit("fcntl - F_SETFL");
41         _exit(EXIT_SUCCESS);
42
43     default:    /* Parent: can see file changes made by child */
44         if (wait(NULL) == -1)
45             errExit("wait");                /* Wait for child exit */
46         printf("Child has exited\n");
47
48         printf("File offset in parent: %lld\n",
49                 (long long) lseek(fd, 0, SEEK_CUR));
50
51         flags = fcntl(fd, F_GETFL);
52         if (flags == -1)
53             errExit("fcntl - F_GETFL");
54         printf("O_APPEND flag in parent is: %s\n",
55                 (flags & O_APPEND) ? "on" : "off");
56         exit(EXIT_SUCCESS);
57     }
58 }
