/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 26-5 */

 1 #include <signal.h>
 2 #include <sys/wait.h>
 3 #include "print_wait_status.h"
 4 #include "curr_time.h"
 5 #include "tlpi_hdr.h"
 6
 7 static volatile int numLiveChildren = 0;
 8                 /* Number of children started but not yet waited on */
 9
10 static void
11 sigchldHandler(int sig)
12 {
13     int status, savedErrno;
14     pid_t childPid;
15
16     /* UNSAFE: This handler uses non-async-signal-safe functions
17        (printf(), printWaitStatus(), currTime(); see Section 21.1.2) */
18
19     savedErrno = errno;         /* In case we modify 'errno' */
20
21     printf("%s handler: Caught SIGCHLD\n", currTime("%T"));
22
23     while ((childPid = waitpid(-1, &status, WNOHANG)) > 0) {
24         printf("%s handler: Reaped child %ld - ", currTime("%T"),
25                 (long) childPid);
26         printWaitStatus(NULL, status);
27         numLiveChildren--;
28     }
29
30     if (childPid == -1 && errno != ECHILD)
31         errMsg("waitpid");
32
33     sleep(5);           /* Artificially lengthen execution of handler */
34     printf("%s handler: returning\n", currTime("%T"));
35
36     errno = savedErrno;
37 }
38
39 int
40 main(int argc, char *argv[])
41 {
42     int j, sigCnt;
43     sigset_t blockMask, emptyMask;
44     struct sigaction sa;
45
46     if (argc < 2 || strcmp(argv[1], "--help") == 0)
47         usageErr("%s child-sleep-time...\n", argv[0]);
48
49     setbuf(stdout, NULL);       /* Disable buffering of stdout */
50
51     sigCnt = 0;
52     numLiveChildren = argc - 1;
53
54     sigemptyset(&sa.sa_mask);
55     sa.sa_flags = 0;
56     sa.sa_handler = sigchldHandler;
57     if (sigaction(SIGCHLD, &sa, NULL) == -1)
58         errExit("sigaction");
59
60     /* Block SIGCHLD to prevent its delivery if a child terminates
61        before the parent commences the sigsuspend() loop below */
62
63     sigemptyset(&blockMask);
64     sigaddset(&blockMask, SIGCHLD);
65     if (sigprocmask(SIG_SETMASK, &blockMask, NULL) == -1)
66         errExit("sigprocmask");
67
68     for (j = 1; j < argc; j++) {
69         switch (fork()) {
70         case -1:
71             errExit("fork");
72
73         case 0:         /* Child - sleeps and then exits */
74             sleep(getInt(argv[j], GN_NONNEG, "child-sleep-time"));
75             printf("%s Child %d (PID=%ld) exiting\n", currTime("%T"),
76                     j, (long) getpid());
77             _exit(EXIT_SUCCESS);
78
79         default:        /* Parent - loops to create next child */
80             break;
81         }
82     }
83
84     /* Parent comes here: wait for SIGCHLD until all children are dead */
85
86     sigemptyset(&emptyMask);
87     while (numLiveChildren > 0) {
88         if (sigsuspend(&emptyMask) == -1 && errno != EINTR)
89             errExit("sigsuspend");
90         sigCnt++;
91     }
92
93     printf("%s All %d children have terminated; SIGCHLD was caught "
94             "%d times\n", currTime("%T"), argc - 1, sigCnt);
95
96     exit(EXIT_SUCCESS);
97 }
