/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 28-3 */

 1 #define _GNU_SOURCE
 2 #include <signal.h>
 3 #include <sys/wait.h>
 4 #include <fcntl.h>
 5 #include <sched.h>
 6 #include "tlpi_hdr.h"
 7
 8 #ifndef CHILD_SIG
 9 #define CHILD_SIG SIGUSR1       /* Signal to be generated on termination
10                                    of cloned child */
11 #endif
12
13 static int                      /* Startup function for cloned child */
14 childFunc(void *arg)
15 {
16     if (close(*((int *) arg)) == -1)
17         errExit("close");
18
19     return 0;                           /* Child terminates now */
20 }
21
22 int
23 main(int argc, char *argv[])
24 {
25     const int STACK_SIZE = 65536;       /* Stack size for cloned child */
26     char *stack;                        /* Start of stack buffer */
27     char *stackTop;                     /* End of stack buffer */
28     int s, fd, flags;
29
30     fd = open("/dev/null", O_RDWR);     /* Child will close this fd */
31     if (fd == -1)
32         errExit("open");
33
34     /* If argc > 1, child shares file descriptor table with parent */
35
36     flags = (argc > 1) ? CLONE_FILES : 0;
37
38     /* Allocate stack for child */
39
40     stack = malloc(STACK_SIZE);
41     if (stack == NULL)
42         errExit("malloc");
43     stackTop = stack + STACK_SIZE;      /* Assume stack grows downward */
44
45     /* Ignore CHILD_SIG, in case it is a signal whose default is to
46        terminate the process; but don't ignore SIGCHLD (which is ignored
47        by default), since that would prevent the creation of a zombie. */
48
49     if (CHILD_SIG != 0 && CHILD_SIG != SIGCHLD)
50         if (signal(CHILD_SIG, SIG_IGN) == SIG_ERR)
51             errExit("signal");
52
53     /* Create child; child commences execution in childFunc() */
54
55     if (clone(childFunc, stackTop, flags | CHILD_SIG, (void *) &fd) == -1)
56         errExit("clone");
57
58     /* Parent falls through to here. Wait for child; __WCLONE is
59        needed for child notifying with signal other than SIGCHLD. */
60
61     if (waitpid(-1, NULL, (CHILD_SIG != SIGCHLD) ? __WCLONE : 0) == -1)
62         errExit("waitpid");
63     printf("child has terminated\n");
64
65     /* Did close() of file descriptor in child affect parent? */
66
67     s = write(fd, "x", 1);
68     if (s == -1 && errno == EBADF)
69         printf("file descriptor %d has been closed\n", fd);
70     else if (s == -1)
71         printf("write() on file descriptor %d failed "
72                 "unexpectedly (%s)\n", fd, strerror(errno));
73     else
74         printf("write() on file descriptor %d succeeded\n", fd);
75
76     exit(EXIT_SUCCESS);
77 }
