/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 53-6 */

 1 #include <semaphore.h>
 2 #include <pthread.h>
 3 #include "tlpi_hdr.h"
 4
 5 static int glob = 0;
 6 static sem_t sem;
 7
 8 static void *                   /* Loop 'arg' times incrementing 'glob' */
 9 threadFunc(void *arg)
10 {
11     int loops = *((int *) arg);
12     int loc, j;
13
14     for (j = 0; j < loops; j++) {
15         if (sem_wait(&sem) == -1)
16             errExit("sem_wait");
17
18         loc = glob;
19         loc++;
20         glob = loc;
21
22         if (sem_post(&sem) == -1)
23             errExit("sem_post");
24     }
25
26     return NULL;
27 }
28
29 int
30 main(int argc, char *argv[])
31 {
32     pthread_t t1, t2;
33     int loops, s;
34
35     loops = (argc > 1) ? getInt(argv[1], GN_GT_0, "num-loops") : 10000000;
36
37     /* Initialize a semaphore with the value 1 */
38
39     if (sem_init(&sem, 0, 1) == -1)
40         errExit("sem_init");
41
42     /* Create two threads that increment 'glob' */
43
44     s = pthread_create(&t1, NULL, threadFunc, &loops);
45     if (s != 0)
46         errExitEN(s, "pthread_create");
47     s = pthread_create(&t2, NULL, threadFunc, &loops);
48     if (s != 0)
49         errExitEN(s, "pthread_create");
50
51     /* Wait for threads to terminate */
52
53     s = pthread_join(t1, NULL);
54     if (s != 0)
55         errExitEN(s, "pthread_join");
56     s = pthread_join(t2, NULL);
57     if (s != 0)
58         errExitEN(s, "pthread_join");
59
60     printf("glob = %d\n", glob);
61     exit(EXIT_SUCCESS);
62 }
