 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 64-2 */
12
13  1 #include <fcntl.h>
14  2 #include <termios.h>
15  3 #include <sys/ioctl.h>
16  4 #include "pty_master_open.h"
17  5 #include "pty_fork.h"                   /* Declares ptyFork() */
18  6 #include "tlpi_hdr.h"
19  7
20  8 #define MAX_SNAME 1000
21  9
22 10 pid_t
23 11 ptyFork(int *masterFd, char *slaveName, size_t snLen,
24 12         const struct termios *slaveTermios, const struct winsize *slaveWS)
25 13 {
26 14     int mfd, slaveFd, savedErrno;
27 15     pid_t childPid;
28 16     char slname[MAX_SNAME];
29 17
30 18     mfd = ptyMasterOpen(slname, MAX_SNAME);
31 19     if (mfd == -1)
32 20         return -1;
33 21
34 22     if (slaveName != NULL) {            /* Return slave name to caller */
35 23         if (strlen(slname) < snLen) {
36 24             strncpy(slaveName, slname, snLen);
37 25
38 26         } else {                        /* 'slaveName' was too small */
39 27             close(mfd);
40 28             errno = EOVERFLOW;
41 29             return -1;
42 30         }
43 31     }
44 32
45 33     childPid = fork();
46 34
47 35     if (childPid == -1) {               /* fork() failed */
48 36         savedErrno = errno;             /* close() might change 'errno' */
49 37         close(mfd);                     /* Don't leak file descriptors */
50 38         errno = savedErrno;
51 39         return -1;
52 40     }
53 41
54 42     if (childPid != 0) {                /* Parent */
55 43         *masterFd = mfd;                /* Only parent gets master fd */
56 44         return childPid;                /* Like parent of fork() */
57 45     }
58 46
59 47     /* Child falls through to here */
60 48
61 49     if (setsid() == -1)                 /* Start a new session */
62 50         err_exit("ptyFork:setsid");
63 51
64 52     close(mfd);                         /* Not needed in child */
65 53
66 54     slaveFd = open(slname, O_RDWR);     /* Becomes controlling tty */
67 55     if (slaveFd == -1)
68 56         err_exit("ptyFork:open-slave");
69 57
70 58 #ifdef TIOCSCTTY                        /* Acquire controlling tty on BSD */
71 59     if (ioctl(slaveFd, TIOCSCTTY, 0) == -1)
72 60         err_exit("ptyFork:ioctl-TIOCSCTTY");
73 61 #endif
74 62
75 63     if (slaveTermios != NULL)           /* Set slave tty attributes */
76 64         if (tcsetattr(slaveFd, TCSANOW, slaveTermios) == -1)
77 65             err_exit("ptyFork:tcsetattr");
78 66
79 67     if (slaveWS != NULL)                /* Set slave tty window size */
80 68         if (ioctl(slaveFd, TIOCSWINSZ, slaveWS) == -1)
81 69             err_exit("ptyFork:ioctl-TIOCSWINSZ");
82 70
83 71     /* Duplicate pty slave to be child's stdin, stdout, and stderr */
84 72
85 73     if (dup2(slaveFd, STDIN_FILENO) != STDIN_FILENO)
86 74         err_exit("ptyFork:dup2-STDIN_FILENO");
87 75     if (dup2(slaveFd, STDOUT_FILENO) != STDOUT_FILENO)
88 76         err_exit("ptyFork:dup2-STDOUT_FILENO");
89 77     if (dup2(slaveFd, STDERR_FILENO) != STDERR_FILENO)
90 78         err_exit("ptyFork:dup2-STDERR_FILENO");
91 79
92 80     if (slaveFd > STDERR_FILENO)        /* Safety check */
93 81         close(slaveFd);                 /* No longer need this fd */
94 82
95 83     return 0;                           /* Like child of fork() */
96 84 }
