/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 22-3 */

 1 #define _GNU_SOURCE
 2 #include <string.h>
 3 #include <signal.h>
 4 #include "tlpi_hdr.h"
 5
 6 static volatile int handlerSleepTime;
 7 static volatile int sigCnt = 0;         /* Number of signals received */
 8 static volatile int allDone = 0;
 9
10 static void             /* Handler for signals established using SA_SIGINFO */
11 siginfoHandler(int sig, siginfo_t *si, void *ucontext)
12 {
13     /* UNSAFE: This handler uses non-async-signal-safe functions
14        (printf()); see Section 21.1.2) */
15
16     /* SIGINT or SIGTERM can be used to terminate program */
17
18     if (sig == SIGINT || sig == SIGTERM) {
19         allDone = 1;
20         return;
21     }
22
23     sigCnt++;
24     printf("caught signal %d\n", sig);
25
26     printf("    si_signo=%d, si_code=%d (%s), ", si->si_signo, si->si_code,
27             (si->si_code == SI_USER) ? "SI_USER" :
28             (si->si_code == SI_QUEUE) ? "SI_QUEUE" : "other");
29     printf("si_value=%d\n", si->si_value.sival_int);
30     printf("    si_pid=%ld, si_uid=%ld\n",
31             (long) si->si_pid, (long) si->si_uid);
32
33     sleep(handlerSleepTime);
34 }
35
36 int
37 main(int argc, char *argv[])
38 {
39     struct sigaction sa;
40     int sig;
41     sigset_t prevMask, blockMask;
42
43     if (argc > 1 && strcmp(argv[1], "--help") == 0)
44         usageErr("%s [block-time [handler-sleep-time]]\n", argv[0]);
45
46     printf("%s: PID is %ld\n", argv[0], (long) getpid());
47
48     handlerSleepTime = (argc > 2) ?
49                 getInt(argv[2], GN_NONNEG, "handler-sleep-time") : 1;
50
51     /* Establish handler for most signals. During execution of the handler,
52        mask all other signals to prevent handlers recursively interrupting
53        each other (which would make the output hard to read). */
54
55     sa.sa_sigaction = siginfoHandler;
56     sa.sa_flags = SA_SIGINFO;
57     sigfillset(&sa.sa_mask);
58
59     for (sig = 1; sig < NSIG; sig++)
60         if (sig != SIGTSTP && sig != SIGQUIT)
61             sigaction(sig, &sa, NULL);
62
63     /* Optionally block signals and sleep, allowing signals to be
64        sent to us before they are unblocked and handled */
65
66     if (argc > 1) {
67         sigfillset(&blockMask);
68         sigdelset(&blockMask, SIGINT);
69         sigdelset(&blockMask, SIGTERM);
70
71         if (sigprocmask(SIG_SETMASK, &blockMask, &prevMask) == -1)
72             errExit("sigprocmask");
73
74         printf("%s: signals blocked - sleeping %s seconds\n", argv[0], argv[1]);
75         sleep(getInt(argv[1], GN_GT_0, "block-time"));
76         printf("%s: sleep complete\n", argv[0]);
77
78         if (sigprocmask(SIG_SETMASK, &prevMask, NULL) == -1)
79             errExit("sigprocmask");
80     }
81
82     while (!allDone)                    /* Wait for incoming signals */
83         pause();
84
85     printf("Caught %d signals\n", sigCnt);
86     exit(EXIT_SUCCESS);
87 }
