/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 21-1 */

 1 #define _XOPEN_SOURCE 600
 2 #include <unistd.h>
 3 #include <signal.h>
 4 #include <string.h>
 5 #include "tlpi_hdr.h"
 6
 7 static char *str2;              /* Set from argv[2] */
 8 static int handled = 0;         /* Counts number of calls to handler */
 9
10 static void
11 handler(int sig)
12 {
13     crypt(str2, "xx");
14     handled++;
15 }
16
17 int
18 main(int argc, char *argv[])
19 {
20     char *cr1;
21     int callNum, mismatch;
22     struct sigaction sa;
23
24     if (argc != 3)
25         usageErr("%s str1 str2\n", argv[0]);
26
27     str2 = argv[2];                      /* Make argv[2] available to handler */
28     cr1 = strdup(crypt(argv[1], "xx"));  /* Copy statically allocated string
29                                             to another buffer */
30     if (cr1 == NULL)
31         errExit("strdup");
32
33     sigemptyset(&sa.sa_mask);
34     sa.sa_flags = 0;
35     sa.sa_handler = handler;
36     if (sigaction(SIGINT, &sa, NULL) == -1)
37         errExit("sigaction");
38
39     /* Repeatedly call crypt() using argv[1]. If interrupted by a
40        signal handler, then the static storage returned by crypt()
41        will be overwritten by the results of encrypting argv[2], and
42        strcmp() will detect a mismatch with the value in 'cr1'. */
43
44     for (callNum = 1, mismatch = 0; ; callNum++) {
45         if (strcmp(crypt(argv[1], "xx"), cr1) != 0) {
46             mismatch++;
47             printf("Mismatch on call %d (mismatch=%d handled=%d)\n",
48                     callNum, mismatch, handled);
49         }
50     }
51 }
