/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 20-7 */

 1 #define _GNU_SOURCE
 2 #include <signal.h>
 3 #include "signal_functions.h"           /* Declaration of printSigset() */
 4 #include "tlpi_hdr.h"
 5
 6 static int sigCnt[NSIG];                /* Counts deliveries of each signal */
 7 static volatile sig_atomic_t gotSigint = 0;
 8                                         /* Set nonzero if SIGINT is delivered */
 9
10 static void
11 handler(int sig)
12 {
13     if (sig == SIGINT)
14         gotSigint = 1;
15     else
16         sigCnt[sig]++;
17 }
18
19 int
20 main(int argc, char *argv[])
21 {
22     int n, numSecs;
23     sigset_t pendingMask, blockingMask, emptyMask;
24
25     printf("%s: PID is %ld\n", argv[0], (long) getpid());
26
27     for (n = 1; n < NSIG; n++)          /* Same handler for all signals */
28         (void) signal(n, handler);      /* Ignore errors */
29
30     /* If a sleep time was specified, temporarily block all signals,
31        sleep (while another process sends us signals), and then
32        display the mask of pending signals and unblock all signals */
33
34     if (argc > 1) {
35         numSecs = getInt(argv[1], GN_GT_0, NULL);
36
37         sigfillset(&blockingMask);
38         if (sigprocmask(SIG_SETMASK, &blockingMask, NULL) == -1)
39             errExit("sigprocmask");
40
41         printf("%s: sleeping for %d seconds\n", argv[0], numSecs);
42         sleep(numSecs);
43
44         if (sigpending(&pendingMask) == -1)
45             errExit("sigpending");
46
47         printf("%s: pending signals are: \n", argv[0]);
48         printSigset(stdout, "\t\t", &pendingMask);
49
50         sigemptyset(&emptyMask);        /* Unblock all signals */
51         if (sigprocmask(SIG_SETMASK, &emptyMask, NULL) == -1)
52             errExit("sigprocmask");
53     }
54
55     while (!gotSigint)                  /* Loop until SIGINT caught */
56         continue;
57
58     for (n = 1; n < NSIG; n++)          /* Display number of signals received */
59         if (sigCnt[n] != 0)
60             printf("%s: signal %d caught %d time%s\n", argv[0], n,
61                     sigCnt[n], (sigCnt[n] == 1) ? "" : "s");
62
63     exit(EXIT_SUCCESS);
64 }
