/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 21-3 */

 1 #define _GNU_SOURCE             /* Get strsignal() declaration from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void
 7 sigsegvHandler(int sig)
 8 {
 9     int x;
10
11     /* UNSAFE: This handler uses non-async-signal-safe functions
12        (printf(), strsignal(), fflush(); see Section 21.1.2) */
13
14     printf("Caught signal %d (%s)\n", sig, strsignal(sig));
15     printf("Top of handler stack near     %10p\n", (void *) &x);
16     fflush(NULL);
17
18     _exit(EXIT_FAILURE);                /* Can't return after SIGSEGV */
19 }
20
21 static void             /* A recursive function that overflows the stack */
22 overflowStack(int callNum)
23 {
24     char a[100000];                     /* Make this stack frame large */
25
26     printf("Call %4d - top of stack near %10p\n", callNum, &a[0]);
27     overflowStack(callNum+1);
28 }
29
30 int
31 main(int argc, char *argv[])
32 {
33     stack_t sigstack;
34     struct sigaction sa;
35     int j;
36
37     printf("Top of standard stack is near %10p\n", (void *) &j);
38
39     /* Allocate alternate stack and inform kernel of its existence */
40
41     sigstack.ss_sp = malloc(SIGSTKSZ);
42     if (sigstack.ss_sp == NULL)
43         errExit("malloc");
44
45     sigstack.ss_size = SIGSTKSZ;
46     sigstack.ss_flags = 0;
47     if (sigaltstack(&sigstack, NULL) == -1)
48         errExit("sigaltstack");
49     printf("Alternate stack is at         %10p-%p\n",
50             sigstack.ss_sp, (char *) sbrk(0) - 1);
51
52     sa.sa_handler = sigsegvHandler;     /* Establish handler for SIGSEGV */
53     sigemptyset(&sa.sa_mask);
54     sa.sa_flags = SA_ONSTACK;           /* Handler uses alternate stack */
55     if (sigaction(SIGSEGV, &sa, NULL) == -1)
56         errExit("sigaction");
57
58     overflowStack(1);
59 }
