/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 22-5 */

 1 #define _GNU_SOURCE     /* Get strsignal() declaration from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include <time.h>
 5 #include "signal_functions.h"           /* Declarations of printSigMask()
 6                                            and printPendingSigs() */
 7 #include "tlpi_hdr.h"
 8
 9 static volatile sig_atomic_t gotSigquit = 0;
10
11 static void
12 handler(int sig)
13 {
14     printf("Caught signal %d (%s)\n", sig, strsignal(sig));
15                                         /* UNSAFE (see Section 21.1.2) */
16     if (sig == SIGQUIT)
17         gotSigquit = 1;
18 }
19
20 int
21 main(int argc, char *argv[])
22 {
23     int loopNum;
24     time_t startTime;
25     sigset_t origMask, blockMask;
26     struct sigaction sa;
27
28     printSigMask(stdout, "Initial signal mask is:\n");
29
30     sigemptyset(&blockMask);
31     sigaddset(&blockMask, SIGINT);
32     sigaddset(&blockMask, SIGQUIT);
33     if (sigprocmask(SIG_BLOCK, &blockMask, &origMask) == -1)
34         errExit("sigprocmask - SIG_BLOCK");
35
36     sigemptyset(&sa.sa_mask);
37     sa.sa_flags = 0;
38     sa.sa_handler = handler;
39     if (sigaction(SIGINT, &sa, NULL) == -1)
40         errExit("sigaction");
41     if (sigaction(SIGQUIT, &sa, NULL) == -1)
42         errExit("sigaction");
43
44     for (loopNum = 1; !gotSigquit; loopNum++) {
45         printf("=== LOOP %d\n", loopNum);
46
47         /* Simulate a critical section by delaying a few seconds */
48
49         printSigMask(stdout, "Starting critical section, signal mask is:\n");
50         for (startTime = time(NULL); time(NULL) < startTime + 4; )
51             continue;                   /* Run for a few seconds elapsed time */
52
53         printPendingSigs(stdout,
54                 "Before sigsuspend() - pending signals:\n");
55         if (sigsuspend(&origMask) == -1 && errno != EINTR)
56             errExit("sigsuspend");
57     }
58
59     if (sigprocmask(SIG_SETMASK, &origMask, NULL) == -1)
60         errExit("sigprocmask - SIG_SETMASK");
61
62     printSigMask(stdout, "=== Exited loop\nRestored signal mask to:\n");
63
64     /* Do other processing... */
65
66     exit(EXIT_SUCCESS);
67 }
