/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 60-4 */

 1 #include <signal.h>
 2 #include <syslog.h>
 3 #include <sys/wait.h>
 4 #include "become_daemon.h"
 5 #include "inet_sockets.h"       /* Declarations of inet*() socket functions */
 6 #include "tlpi_hdr.h"
 7
 8 #define SERVICE "echo"          /* Name of TCP service */
 9 #define BUF_SIZE 4096
10
11 static void             /* SIGCHLD handler to reap dead child processes */
12 grimReaper(int sig)
13 {
14     int savedErrno;             /* Save 'errno' in case changed here */
15
16     savedErrno = errno;
17     while (waitpid(-1, NULL, WNOHANG) > 0)
18         continue;
19     errno = savedErrno;
20 }
21
22 /* Handle a client request: copy socket input back to socket */
23
24 static void
25 handleRequest(int cfd)
26 {
27     char buf[BUF_SIZE];
28     ssize_t numRead;
29
30     while ((numRead = read(cfd, buf, BUF_SIZE)) > 0) {
31         if (write(cfd, buf, numRead) != numRead) {
32             syslog(LOG_ERR, "write() failed: %s", strerror(errno));
33             exit(EXIT_FAILURE);
34         }
35     }
36
37     if (numRead == -1) {
38         syslog(LOG_ERR, "Error from read(): %s", strerror(errno));
39         exit(EXIT_FAILURE);
40     }
41 }
42
43 int
44 main(int argc, char *argv[])
45 {
46     int lfd, cfd;               /* Listening and connected sockets */
47     struct sigaction sa;
48
49     if (becomeDaemon(0) == -1)
50         errExit("becomeDaemon");
51
52     sigemptyset(&sa.sa_mask);
53     sa.sa_flags = SA_RESTART;
54     sa.sa_handler = grimReaper;
55     if (sigaction(SIGCHLD, &sa, NULL) == -1) {
56         syslog(LOG_ERR, "Error from sigaction(): %s", strerror(errno));
57         exit(EXIT_FAILURE);
58     }
59
60     lfd = inetListen(SERVICE, 10, NULL);
61     if (lfd == -1) {
62         syslog(LOG_ERR, "Could not create server socket (%s)", strerror(errno));
63         exit(EXIT_FAILURE);
64     }
65
66     for (;;) {
67         cfd = accept(lfd, NULL, 0);     /* Wait for connection */
68         if (cfd == -1) {
69             syslog(LOG_ERR, "Failure in accept(): %s", strerror(errno));
70             exit(EXIT_FAILURE);
71         }
72
73         /* Handle each client request in a new child process */
74
75         switch (fork()) {
76         case -1:
77             syslog(LOG_ERR, "Can't create child (%s)", strerror(errno));
78             close(cfd);                 /* Give up on this client */
79             break;                      /* May be temporary; try next client */
80
81         case 0:                         /* Child */
82             close(lfd);                 /* Unneeded copy of listening socket */
83             handleRequest(cfd);
84             _exit(EXIT_SUCCESS);
85
86         default:                        /* Parent */
87             close(cfd);                 /* Unneeded copy of connected socket */
88             break;                      /* Loop to accept next connection */
89         }
90     }
91 }
