/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 59-6 */

  1 #define _BSD_SOURCE             /* To get definitions of NI_MAXHOST and
  2                                    NI_MAXSERV from <netdb.h> */
  3 #include <netdb.h>
  4 #include "is_seqnum.h"
  5
  6 #define BACKLOG 50
  7
  8 int
  9 main(int argc, char *argv[])
 10 {
 11     uint32_t seqNum;
 12     char reqLenStr[INT_LEN];            /* Length of requested sequence */
 13     char seqNumStr[INT_LEN];            /* Start of granted sequence */
 14     struct sockaddr_storage claddr;
 15     int lfd, cfd, optval, reqLen;
 16     socklen_t addrlen;
 17     struct addrinfo hints;
 18     struct addrinfo *result, *rp;
 19 #define ADDRSTRLEN (NI_MAXHOST + NI_MAXSERV + 10)
 20     char addrStr[ADDRSTRLEN];
 21     char host[NI_MAXHOST];
 22     char service[NI_MAXSERV];
 23
 24     if (argc > 1 && strcmp(argv[1], "--help") == 0)
 25         usageErr("%s [init-seq-num]\n", argv[0]);
 26
 27     seqNum = (argc > 1) ? getInt(argv[1], 0, "init-seq-num") : 0;
 28
 29     if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
 30         errExit("signal");
 31
 32     /* Call getaddrinfo() to obtain a list of addresses that
 33        we can try binding to */
 34
 35     memset(&hints, 0, sizeof(struct addrinfo));
 36     hints.ai_canonname = NULL;
 37     hints.ai_addr = NULL;
 38     hints.ai_next = NULL;
 39     hints.ai_socktype = SOCK_STREAM;
 40     hints.ai_family = AF_UNSPEC;        /* Allows IPv4 or IPv6 */
 41     hints.ai_flags = AI_PASSIVE | AI_NUMERICSERV;
 42                         /* Wildcard IP address; service name is numeric */
 43
 44     if (getaddrinfo(NULL, PORT_NUM, &hints, &result) != 0)
 45         errExit("getaddrinfo");
 46
 47     /* Walk through returned list until we find an address structure
 48        that can be used to successfully create and bind a socket */
 49
 50     optval = 1;
 51     for (rp = result; rp != NULL; rp = rp->ai_next) {
 52         lfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
 53         if (lfd == -1)
 54             continue;                   /* On error, try next address */
 55
 56         if (setsockopt(lfd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval))
 57                 == -1)
 58              errExit("setsockopt");
 59
 60         if (bind(lfd, rp->ai_addr, rp->ai_addrlen) == 0)
 61             break;                      /* Success */
 62
 63         /* bind() failed: close this socket and try next address */
 64
 65         close(lfd);
 66     }
 67
 68     if (rp == NULL)
 69         fatal("Could not bind socket to any address");
 70
 71     if (listen(lfd, BACKLOG) == -1)
 72         errExit("listen");
 73
 74     freeaddrinfo(result);
 75
 76     for (;;) {                  /* Handle clients iteratively */
 77
 78         /* Accept a client connection, obtaining client's address */
 79
 80         addrlen = sizeof(struct sockaddr_storage);
 81         cfd = accept(lfd, (struct sockaddr *) &claddr, &addrlen);
 82         if (cfd == -1) {
 83             errMsg("accept");
 84             continue;
 85         }
 86
 87         if (getnameinfo((struct sockaddr *) &claddr, addrlen,
 88                     host, NI_MAXHOST, service, NI_MAXSERV, 0) == 0)
 89             snprintf(addrStr, ADDRSTRLEN, "(%s, %s)", host, service);
 90         else
 91             snprintf(addrStr, ADDRSTRLEN, "(?UNKNOWN?)");
 92         printf("Connection from %s\n", addrStr);
 93
 94         /* Read client request, send sequence number back */
 95
 96         if (readLine(cfd, reqLenStr, INT_LEN) <= 0) {
 97             close(cfd);
 98             continue;                   /* Failed read; skip request */
 99         }
100
101         reqLen = atoi(reqLenStr);
102         if (reqLen <= 0) {              /* Watch for misbehaving clients */
103             close(cfd);
104             continue;                   /* Bad request; skip it */
105         }
106
107         snprintf(seqNumStr, INT_LEN, "%d\n", seqNum);
108         if (write(cfd, &seqNumStr, strlen(seqNumStr)) != strlen(seqNumStr))
109             fprintf(stderr, "Error on write");
110
111         seqNum += reqLen;               /* Update sequence number */
112
113         if (close(cfd) == -1)           /* Close connection */
114             errMsg("close");
115     }
116 }
