/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 59-2:b */

 1 #include "is_seqnum_v2.h"
 2
 3 int
 4 main(int argc, char *argv[])
 5 {
 6     uint32_t seqNum;
 7     char reqLenStr[INT_LEN];            /* Length of requested sequence */
 8     char seqNumStr[INT_LEN];            /* Start of granted sequence */
 9     struct sockaddr *claddr;
10     int lfd, cfd, reqLen;
11     socklen_t addrlen, alen;
12     char addrStr[IS_ADDR_STR_LEN];
13
14     if (argc > 1 && strcmp(argv[1], "--help") == 0)
15         usageErr("%s [init-seq-num]\n", argv[0]);
16
17     seqNum = (argc > 1) ? getInt(argv[1], 0, "init-seq-num") : 0;
18
19     if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
20         errExit("signal");
21
22     lfd = inetListen(PORT_NUM_STR, 5, &addrlen);
23     if (lfd == -1)
24         fatal("inetListen() failed");
25
26     claddr = malloc(addrlen);
27     if (claddr == NULL)
28         errExit("malloc");
29
30     for (;;) {                  /* Handle clients iteratively */
31
32         /* Accept a client connection, obtaining client's address */
33
34         alen = addrlen;
35         cfd = accept(lfd, (struct sockaddr *) claddr, &alen);
36         if (cfd == -1) {
37             errMsg("accept");
38             continue;
39         }
40
41         printf("Connection from %s\n", inetAddressStr(claddr, alen,
42                         addrStr, IS_ADDR_STR_LEN));
43
44         /* Read client request, send sequence number back */
45
46         if (readLine(cfd, reqLenStr, INT_LEN) <= 0) {
47             close(cfd);
48             continue;                   /* Failed read; skip request */
49         }
50
51         reqLen = atoi(reqLenStr);
52         if (reqLen <= 0) {              /* Watch for misbehaving clients */
53             close(cfd);
54             continue;                   /* Bad request; skip it */
55         }
56
57         snprintf(seqNumStr, INT_LEN, "%d\n", seqNum);
58         if (write(cfd, &seqNumStr, strlen(seqNumStr)) != strlen(seqNumStr))
59             fprintf(stderr, "Error on write");
60
61         seqNum += reqLen;               /* Update sequence number */
62
63         if (close(cfd) == -1)           /* Close connection */
64             errMsg("close");
65     }
66 }
