 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 59-1 */
12
13  1 #include <unistd.h>
14  2 #include <errno.h>
15  3 #include "read_line.h"                  /* Declaration of readLine() */
16  4
17  5 ssize_t
18  6 readLine(int fd, void *buffer, size_t n)
19  7 {
20  8     ssize_t numRead;                    /* # of bytes fetched by last read() */
21  9     size_t totRead;                     /* Total bytes read so far */
22 10     char *buf;
23 11     char ch;
24 12
25 13     if (n <= 0 || buffer == NULL) {
26 14         errno = EINVAL;
27 15         return -1;
28 16     }
29 17
30 18     buf = buffer;                       /* No pointer arithmetic on "void *" */
31 19
32 20     totRead = 0;
33 21     for (;;) {
34 22         numRead = read(fd, &ch, 1);
35 23
36 24         if (numRead == -1) {
37 25             if (errno == EINTR)         /* Interrupted --> restart read() */
38 26                 continue;
39 27             else
40 28                 return -1;              /* Some other error */
41 29
42 30         } else if (numRead == 0) {      /* EOF */
43 31             if (totRead == 0)           /* No bytes read; return 0 */
44 32                 return 0;
45 33             else                        /* Some bytes read; add '\0' */
46 34                 break;
47 35
48 36         } else {                        /* 'numRead' must be 1 if we get here */
49 37             if (totRead < n - 1) {      /* Discard > (n - 1) bytes */
50 38                 totRead++;
51 39                 *buf++ = ch;
52 40             }
53 41
54 42             if (ch == '\n')
55 43                 break;
56 44         }
57 45     }
58 46
59 47     *buf = '\0';
60 48     return totRead;
61 49 }
