/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 61 */

  1 #include "scm_cred.h"
  2
  3 int
  4 main(int argc, char *argv[])
  5 {
  6     struct msghdr msgh;
  7     struct iovec iov;
  8     struct ucred *ucredp, ucred;
  9     int data, lfd, sfd, optval, opt;
 10     ssize_t nr;
 11     Boolean useDatagramSocket;
 12     union {
 13         struct cmsghdr cmh;
 14         char   control[CMSG_SPACE(sizeof(struct ucred))];
 15                         /* Space large enough to hold a ucred structure */
 16     } control_un;
 17     struct cmsghdr *cmhp;
 18     socklen_t len;
 19
 20     /* Parse command-line arguments */
 21
 22     useDatagramSocket = FALSE;
 23
 24     while ((opt = getopt(argc, argv, "d")) != -1) {
 25         switch (opt) {
 26         case 'd':
 27             useDatagramSocket = TRUE;
 28             break;
 29
 30         default:
 31             usageErr("%s [-d]\n"
 32                     "        -d    use datagram socket\n", argv[0]);
 33         }
 34     }
 35
 36     /* Create socket bound to well-known address */
 37
 38     if (remove(SOCK_PATH) == -1 && errno != ENOENT)
 39         errExit("remove-%s", SOCK_PATH);
 40
 41     if (useDatagramSocket) {
 42         printf("Receiving via datagram socket\n");
 43         sfd = unixBind(SOCK_PATH, SOCK_DGRAM);
 44         if (sfd == -1)
 45             errExit("unixBind");
 46
 47     } else {
 48         printf("Receiving via stream socket\n");
 49         lfd = unixListen(SOCK_PATH, 5);
 50         if (lfd == -1)
 51             errExit("unixListen");
 52
 53         sfd = accept(lfd, NULL, 0);
 54         if (sfd == -1)
 55             errExit("accept");
 56     }
 57
 58     /* We must set the SO_PASSCRED socket option in order to receive
 59        credentials */
 60
 61     optval = 1;
 62     if (setsockopt(sfd, SOL_SOCKET, SO_PASSCRED, &optval, sizeof(optval)) == -1)
 63         errExit("setsockopt");
 64
 65     /* Set 'control_un' to describe ancillary data that we want to receive */
 66
 67     control_un.cmh.cmsg_len = CMSG_LEN(sizeof(struct ucred));
 68     control_un.cmh.cmsg_level = SOL_SOCKET;
 69     control_un.cmh.cmsg_type = SCM_CREDENTIALS;
 70
 71     /* Set 'msgh' fields to describe 'control_un' */
 72
 73     msgh.msg_control = control_un.control;
 74     msgh.msg_controllen = sizeof(control_un.control);
 75
 76     /* Set fields of 'msgh' to point to buffer used to receive (real)
 77        data read by recvmsg() */
 78
 79     msgh.msg_iov = &iov;
 80     msgh.msg_iovlen = 1;
 81     iov.iov_base = &data;
 82     iov.iov_len = sizeof(int);
 83
 84     msgh.msg_name = NULL;               /* We don't need address of peer */
 85     msgh.msg_namelen = 0;
 86
 87     /* Receive real plus ancillary data */
 88
 89     nr = recvmsg(sfd, &msgh, 0);
 90     if (nr == -1)
 91         errExit("recvmsg");
 92     printf("recvmsg() returned %ld\n", (long) nr);
 93
 94     if (nr > 0)
 95         printf("Received data = %d\n", data);
 96
 97     /* Extract credentials information from received ancillary data */
 98
 99     cmhp = CMSG_FIRSTHDR(&msgh);
100     if (cmhp == NULL || cmhp->cmsg_len != CMSG_LEN(sizeof(struct ucred)))
101         fatal("bad cmsg header / message length");
102     if (cmhp->cmsg_level != SOL_SOCKET)
103         fatal("cmsg_level != SOL_SOCKET");
104     if (cmhp->cmsg_type != SCM_CREDENTIALS)
105         fatal("cmsg_type != SCM_CREDENTIALS");
106
107     ucredp = (struct ucred *) CMSG_DATA(cmhp);
108     printf("Received credentials pid=%ld, uid=%ld, gid=%ld\n",
109                 (long) ucredp->pid, (long) ucredp->uid, (long) ucredp->gid);
110
111     /* The Linux-specific, read-only SO_PEERCRED socket option returns
112        credential information about the peer, as described in socket(7) */
113
114     len = sizeof(struct ucred);
115     if (getsockopt(sfd, SOL_SOCKET, SO_PEERCRED, &ucred, &len) == -1)
116         errExit("getsockopt");
117
118     printf("Credentials from SO_PEERCRED: pid=%ld, euid=%ld, egid=%ld\n",
119             (long) ucred.pid, (long) ucred.uid, (long) ucred.gid);
120
121     exit(EXIT_SUCCESS);
122 }
