/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 61 */

  1 #include "scm_cred.h"
  2
  3 int
  4 main(int argc, char *argv[])
  5 {
  6     struct msghdr msgh;
  7     struct iovec iov;
  8     int data, sfd, opt;
  9     ssize_t ns;
 10     Boolean useDatagramSocket, noExplicit;
 11     union {
 12         struct cmsghdr cmh;
 13         char   control[CMSG_SPACE(sizeof(struct ucred))];
 14                         /* Space large enough to hold a ucred structure */
 15     } control_un;
 16     struct cmsghdr *cmhp;
 17
 18     /* Parse command-line arguments */
 19
 20     useDatagramSocket = FALSE;
 21     noExplicit = FALSE;
 22
 23     while ((opt = getopt(argc, argv, "dn")) != -1) {
 24         switch (opt) {
 25         case 'd':
 26             useDatagramSocket = TRUE;
 27             break;
 28
 29         case 'n':
 30             noExplicit = TRUE;
 31             break;
 32
 33         default:
 34             usageErr("%s [-d] [-n] [data [PID [UID [GID]]]]\n"
 35                     "        -d    use datagram socket\n"
 36                     "        -n    don't construct explicit "
 37                                   "credentials structure\n", argv[0]);
 38         }
 39     }
 40
 41     /* On Linux, we must transmit at least 1 byte of real data in
 42        order to send ancillary data */
 43
 44     msgh.msg_iov = &iov;
 45     msgh.msg_iovlen = 1;
 46     iov.iov_base = &data;
 47     iov.iov_len = sizeof(int);
 48
 49     /* Data is optionally taken from command line */
 50
 51     data = (argc > optind) ? atoi(argv[optind]) : 12345;
 52
 53     /* Don't need to specify destination address, because we use
 54        connect() below */
 55
 56     msgh.msg_name = NULL;
 57     msgh.msg_namelen = 0;
 58
 59     if (noExplicit) {
 60
 61         /* Don't construct an explicit credentials structure. (It
 62            is not necessary to do so, if we just want the receiver to
 63            receive our real credentials.) */
 64
 65         printf("Not explicitly sending a credentials structure\n");
 66         msgh.msg_control = NULL;
 67         msgh.msg_controllen = 0;
 68
 69     } else {
 70         struct ucred *ucp;
 71
 72         /* Set 'msgh' fields to describe 'control_un' */
 73
 74         msgh.msg_control = control_un.control;
 75         msgh.msg_controllen = sizeof(control_un.control);
 76
 77         /* Set message header to describe ancillary data that we want to send */
 78
 79         cmhp = CMSG_FIRSTHDR(&msgh);
 80         cmhp->cmsg_len = CMSG_LEN(sizeof(struct ucred));
 81         cmhp->cmsg_level = SOL_SOCKET;
 82         cmhp->cmsg_type = SCM_CREDENTIALS;
 83         ucp = (struct ucred *) CMSG_DATA(cmhp);
 84
 85         /*
 86         We could rewrite the preceding as:
 87
 88         control_un.cmh.cmsg_len = CMSG_LEN(sizeof(struct ucred));
 89         control_un.cmh.cmsg_level = SOL_SOCKET;
 90         control_un.cmh.cmsg_type = SCM_CREDENTIALS;
 91         ucp = (struct ucred *) CMSG_DATA(CMSG_FIRSTHDR(&msgh));
 92         */
 93
 94         /* Use sender's own PID, real UID, and real GID, unless
 95            alternate values were supplied on the command line */
 96
 97         ucp->pid = (argc > optind + 1 && strcmp(argv[optind + 1], "-") != 0) ?
 98                         atoi(argv[optind + 1]) : getpid();
 99         ucp->uid = (argc > optind + 2 && strcmp(argv[optind + 2], "-") != 0) ?
100                         atoi(argv[optind + 2]) : getuid();
101         ucp->gid = (argc > optind + 3 && strcmp(argv[optind + 3], "-") != 0) ?
102                         atoi(argv[optind + 3]) : getgid();
103
104         printf("Send credentials pid=%ld, uid=%ld, gid=%ld\n",
105                 (long) ucp->pid, (long) ucp->uid, (long) ucp->gid);
106     }
107
108     sfd = unixConnect(SOCK_PATH, useDatagramSocket ? SOCK_DGRAM : SOCK_STREAM);
109     if (sfd == -1)
110         errExit("unixConnect");
111
112     ns = sendmsg(sfd, &msgh, 0);
113     if (ns == -1)
114         errExit("sendmsg");
115     printf("sendmsg() returned %ld\n", (long) ns);
116
117     exit(EXIT_SUCCESS);
118 }
