/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 46-7 */

 1 #include <sys/types.h>
 2 #include <sys/msg.h>
 3 #include <sys/stat.h>
 4 #include <stddef.h>                     /* For definition of offsetof() */
 5 #include <limits.h>
 6 #include <fcntl.h>
 7 #include <signal.h>
 8 #include <sys/wait.h>
 9 #include "tlpi_hdr.h"
10 
11 #define SERVER_KEY 0x1aaaaaa1           /* Key for server's message queue */
12 
13 struct requestMsg {                     /* Requests (client to server) */
14     long mtype;                         /* Unused */
15     int  clientId;                      /* ID of client's message queue */
16     char pathname[PATH_MAX];            /* File to be returned */
17 };
18 
19 /* REQ_MSG_SIZE computes size of 'mtext' part of 'requestMsg' structure.
20    We use offsetof() to handle the possibility that there are padding
21    bytes between the 'clientId' and 'pathname' fields. */
22 
23 #define REQ_MSG_SIZE (offsetof(struct requestMsg, pathname) - \
24                       offsetof(struct requestMsg, clientId) + PATH_MAX)
25 
26 #define RESP_MSG_SIZE 8192
27 
28 struct responseMsg {                    /* Responses (server to client) */
29     long mtype;                         /* One of RESP_MT_* values below */
30     char data[RESP_MSG_SIZE];           /* File content / response message */
31 };
32 
33 /* Types for response messages sent from server to client */
34 
35 #define RESP_MT_FAILURE 1               /* File couldn't be opened */
36 #define RESP_MT_DATA    2               /* Message contains file data */
37 #define RESP_MT_END     3               /* File data complete */
