/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 46-9 */

 1 #include "svmsg_file.h"
 2
 3 static int clientId;
 4
 5 static void
 6 removeQueue(void)
 7 {
 8     if (msgctl(clientId, IPC_RMID, NULL) == -1)
 9         errExit("msgctl");
10 }
11
12 int
13 main(int argc, char *argv[])
14 {
15     struct requestMsg req;
16     struct responseMsg resp;
17     int serverId, numMsgs;
18     ssize_t msgLen, totBytes;
19
20     if (argc != 2 || strcmp(argv[1], "--help") == 0)
21         usageErr("%s pathname\n", argv[0]);
22
23     if (strlen(argv[1]) > sizeof(req.pathname) - 1)
24         cmdLineErr("pathname too long (max: %ld bytes)\n",
25                 (long) sizeof(req.pathname) - 1);
26
27     /* Get server's queue identifier; create queue for response */
28
29     serverId = msgget(SERVER_KEY, S_IWUSR);
30     if (serverId == -1)
31         errExit("msgget - server message queue");
32
33     clientId = msgget(IPC_PRIVATE, S_IRUSR | S_IWUSR | S_IWGRP);
34     if (clientId == -1)
35         errExit("msgget - client message queue");
36
37     if (atexit(removeQueue) != 0)
38         errExit("atexit");
39
40     /* Send message asking for file named in argv[1] */
41
42     req.mtype = 1;                      /* Any type will do */
43     req.clientId = clientId;
44     strncpy(req.pathname, argv[1], sizeof(req.pathname) - 1);
45     req.pathname[sizeof(req.pathname) - 1] = '\0';
46                                         /* Ensure string is terminated */
47
48     if (msgsnd(serverId, &req, REQ_MSG_SIZE, 0) == -1)
49         errExit("msgsnd");
50
51     /* Get first response, which may be failure notification */
52
53     msgLen = msgrcv(clientId, &resp, RESP_MSG_SIZE, 0, 0);
54     if (msgLen == -1)
55         errExit("msgrcv");
56
57     if (resp.mtype == RESP_MT_FAILURE) {
58         printf("%s\n", resp.data);      /* Display msg from server */
59         exit(EXIT_FAILURE);
60     }
61
62     /* File was opened successfully by server; process messages
63        (including the one already received) containing file data */
64
65     totBytes = msgLen;                  /* Count first message */
66     for (numMsgs = 1; resp.mtype == RESP_MT_DATA; numMsgs++) {
67         msgLen = msgrcv(clientId, &resp, RESP_MSG_SIZE, 0, 0);
68         if (msgLen == -1)
69             errExit("msgrcv");
70
71         totBytes += msgLen;
72     }
73
74     printf("Received %ld bytes (%d messages)\n", (long) totBytes, numMsgs);
75
76     exit(EXIT_SUCCESS);
77 }
