/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 46-8 */

 1 #include "svmsg_file.h"
 2
 3 static void             /* SIGCHLD handler */
 4 grimReaper(int sig)
 5 {
 6     int savedErrno;
 7
 8     savedErrno = errno;                 /* waitpid() might change 'errno' */
 9     while (waitpid(-1, NULL, WNOHANG) > 0)
10         continue;
11     errno = savedErrno;
12 }
13
14 static void             /* Executed in child process: serve a single client */
15 serveRequest(const struct requestMsg *req)
16 {
17     int fd;
18     ssize_t numRead;
19     struct responseMsg resp;
20
21     fd = open(req->pathname, O_RDONLY);
22     if (fd == -1) {                     /* Open failed: send error text */
23         resp.mtype = RESP_MT_FAILURE;
24         snprintf(resp.data, sizeof(resp.data), "%s", "Couldn't open");
25         msgsnd(req->clientId, &resp, strlen(resp.data) + 1, 0);
26         exit(EXIT_FAILURE);             /* and terminate */
27     }
28
29     /* Transmit file contents in messages with type RESP_MT_DATA. We don't
30        diagnose read() and msgsnd() errors since we can't notify client. */
31
32     resp.mtype = RESP_MT_DATA;
33     while ((numRead = read(fd, resp.data, RESP_MSG_SIZE)) > 0)
34         if (msgsnd(req->clientId, &resp, numRead, 0) == -1)
35             break;
36
37     /* Send a message of type RESP_MT_END to signify end-of-file */
38
39     resp.mtype = RESP_MT_END;
40     msgsnd(req->clientId, &resp, 0, 0);         /* Zero-length mtext */
41 }
42
43 int
44 main(int argc, char *argv[])
45 {
46     struct requestMsg req;
47     pid_t pid;
48     ssize_t msgLen;
49     int serverId;
50     struct sigaction sa;
51
52     /* Create server message queue */
53
54     serverId = msgget(SERVER_KEY, IPC_CREAT | IPC_EXCL |
55                             S_IRUSR | S_IWUSR | S_IWGRP);
56     if (serverId == -1)
57         errExit("msgget");
58
59     /* Establish SIGCHLD handler to reap terminated children */
60
61     sigemptyset(&sa.sa_mask);
62     sa.sa_flags = SA_RESTART;
63     sa.sa_handler = grimReaper;
64     if (sigaction(SIGCHLD, &sa, NULL) == -1)
65         errExit("sigaction");
66
67     /* Read requests, handle each in a separate child process */
68
69     for (;;) {
70         msgLen = msgrcv(serverId, &req, REQ_MSG_SIZE, 0, 0);
71         if (msgLen == -1) {
72             if (errno == EINTR)         /* Interrupted by SIGCHLD handler? */
73                 continue;               /* ... then restart msgrcv() */
74             errMsg("msgrcv");           /* Some other error */
75             break;                      /* ... so terminate loop */
76         }
77
78         pid = fork();                   /* Create child process */
79         if (pid == -1) {
80             errMsg("fork");
81             break;
82         }
83
84         if (pid == 0) {                 /* Child handles request */
85             serveRequest(&req);
86             _exit(EXIT_SUCCESS);
87         }
88
89         /* Parent loops to receive next client request */
90     }
91
92     /* If msgrcv() or fork() fails, remove server MQ and exit */
93
94     if (msgctl(serverId, IPC_RMID, NULL) == -1)
95         errExit("msgctl");
96     exit(EXIT_SUCCESS);
97 }
