/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 48 */

 1 #include <sys/types.h>
 2 #include <sys/ipc.h>
 3 #include <sys/shm.h>
 4 #include <sys/stat.h>
 5 #include "tlpi_hdr.h"
 6
 7 static void
 8 usageError(const char *progName, const char *msg)
 9 {
10     if (msg != NULL)
11         fprintf(stderr, "%s", msg);
12     fprintf(stderr, "Usage: %s [-cx] {-f pathname | -k key | -p} "
13                             "seg-size [octal-perms]\n", progName);
14     fprintf(stderr, "    -c           Use IPC_CREAT flag\n");
15     fprintf(stderr, "    -x           Use IPC_EXCL flag\n");
16     fprintf(stderr, "    -f pathname  Generate key using ftok()\n");
17     fprintf(stderr, "    -k key       Use 'key' as key\n");
18     fprintf(stderr, "    -p           Use IPC_PRIVATE key\n");
19     exit(EXIT_FAILURE);
20 }
21
22 int
23 main(int argc, char *argv[])
24 {
25     int numKeyFlags;            /* Counts -f, -k, and -p options */
26     int flags, shmid, segSize;
27     unsigned int perms;
28     long lkey;
29     key_t key;
30     int opt;                    /* Option character from getopt() */
31
32     /* Parse command-line options and arguments */
33
34     numKeyFlags = 0;
35     flags = 0;
36
37     while ((opt = getopt(argc, argv, "cf:k:px")) != -1) {
38         switch (opt) {
39         case 'c':
40             flags |= IPC_CREAT;
41             break;
42
43         case 'f':               /* -f pathname */
44             key = ftok(optarg, 1);
45             if (key == -1)
46                 errExit("ftok");
47             numKeyFlags++;
48             break;
49
50         case 'k':               /* -k key (octal, decimal or hexadecimal) */
51             if (sscanf(optarg, "%li", &lkey) != 1)
52                 cmdLineErr("-k option requires a numeric argument\n");
53             key = lkey;
54             numKeyFlags++;
55             break;
56
57         case 'p':
58             key = IPC_PRIVATE;
59             numKeyFlags++;
60             break;
61
62         case 'x':
63             flags |= IPC_EXCL;
64             break;
65
66         default:
67             usageError(argv[0], NULL);
68         }
69     }
70
71     if (numKeyFlags != 1)
72         usageError(argv[0], "Exactly one of the options -f, -k, "
73                             "or -p must be supplied\n");
74
75     if (optind >= argc)
76         usageError(argv[0], "Size of segment must be specified\n");
77
78     segSize = getLong(argv[optind], GN_ANY_BASE, "seg-size");
79
80     perms = (argc <= optind + 1) ? (S_IRUSR | S_IWUSR) :
81                 getInt(argv[optind + 1], GN_BASE_8, "octal-perms");
82
83     shmid = shmget(key, segSize, flags | perms);
84     if (shmid == -1)
85         errExit("shmget");
86
87     printf("%d\n", shmid);      /* On success, display shared mem. id */
88     exit(EXIT_SUCCESS);
89 }
