/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 48-3 */

 1 #include "svshm_xfr.h"
 2
 3 int
 4 main(int argc, char *argv[])
 5 {
 6     int semid, shmid, xfrs, bytes;
 7     struct shmseg *shmp;
 8
 9     /* Get IDs for semaphore set and shared memory created by writer */
10
11     semid = semget(SEM_KEY, 0, 0);
12     if (semid == -1)
13         errExit("semget");
14
15     shmid  = shmget(SHM_KEY, 0, 0);
16     if (shmid == -1)
17         errExit("shmget");
18
19     shmp = shmat(shmid, NULL, SHM_RDONLY);
20     if (shmp == (void *) -1)
21         errExit("shmat");
22
23     /* Transfer blocks of data from shared memory to stdout */
24
25     for (xfrs = 0, bytes = 0; ; xfrs++) {
26         if (reserveSem(semid, READ_SEM) == -1)          /* Wait for our turn */
27             errExit("reserveSem");
28
29         if (shmp->cnt == 0)                     /* Writer encountered EOF */
30             break;
31         bytes += shmp->cnt;
32
33         if (write(STDOUT_FILENO, shmp->buf, shmp->cnt) != shmp->cnt)
34             fatal("partial/failed write");
35
36         if (releaseSem(semid, WRITE_SEM) == -1)         /* Give writer a turn */
37             errExit("releaseSem");
38     }
39
40     if (shmdt(shmp) == -1)
41         errExit("shmdt");
42
43     /* Give writer one more turn, so it can clean up */
44
45     if (releaseSem(semid, WRITE_SEM) == -1)
46         errExit("releaseSem");
47
48     fprintf(stderr, "Received %d bytes (%d xfrs)\n", bytes, xfrs);
49     exit(EXIT_SUCCESS);
50 }
