/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 30 */

  1 #include <time.h>
  2 #include <pthread.h>
  3 #include "tlpi_hdr.h"
  4
  5 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
  6 static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
  7
  8 static int avail = 0;
  9
 10 static void *
 11 threadFunc(void *arg)
 12 {
 13     int cnt = atoi((char *) arg);
 14     int s, j;
 15
 16     for (j = 0; j < cnt; j++) {
 17         sleep(1);
 18
 19         /* Code to produce a unit omitted */
 20
 21         s = pthread_mutex_lock(&mtx);
 22         if (s != 0)
 23             errExitEN(s, "pthread_mutex_lock");
 24
 25         avail++;        /* Let consumer know another unit is available */
 26
 27         s = pthread_mutex_unlock(&mtx);
 28         if (s != 0)
 29             errExitEN(s, "pthread_mutex_unlock");
 30
 31         s = pthread_cond_signal(&cond);         /* Wake sleeping consumer */
 32         if (s != 0)
 33             errExitEN(s, "pthread_cond_signal");
 34     }
 35
 36     return NULL;
 37 }
 38
 39 int
 40 main(int argc, char *argv[])
 41 {
 42     pthread_t tid;
 43     int s, j;
 44     int totRequired;            /* Total number of units that all threads
 45                                    will produce */
 46     int numConsumed;            /* Total units so far consumed */
 47     Boolean done;
 48     time_t t;
 49
 50     t = time(NULL);
 51
 52     /* Create all threads */
 53
 54     totRequired = 0;
 55     for (j = 1; j < argc; j++) {
 56         totRequired += atoi(argv[j]);
 57
 58         s = pthread_create(&tid, NULL, threadFunc, argv[j]);
 59         if (s != 0)
 60             errExitEN(s, "pthread_create");
 61     }
 62
 63     /* Use a polling loop to check for available units */
 64
 65     numConsumed = 0;
 66     done = FALSE;
 67
 68     for (;;) {
 69         s = pthread_mutex_lock(&mtx);
 70         if (s != 0)
 71             errExitEN(s, "pthread_mutex_lock");
 72
 73         while (avail == 0) {            /* Wait for something to consume */
 74             s = pthread_cond_wait(&cond, &mtx);
 75             if (s != 0)
 76                 errExitEN(s, "pthread_cond_wait");
 77         }
 78
 79         /* At this point, 'mtx' is locked... */
 80
 81         while (avail > 0) {             /* Consume all available units */
 82
 83             /* Do something with produced unit */
 84
 85             numConsumed ++;
 86             avail--;
 87             printf("T=%ld: numConsumed=%d\n", (long) (time(NULL) - t),
 88                     numConsumed);
 89
 90             done = numConsumed >= totRequired;
 91         }
 92
 93         s = pthread_mutex_unlock(&mtx);
 94         if (s != 0)
 95             errExitEN(s, "pthread_mutex_unlock");
 96
 97         if (done)
 98             break;
 99
100         /* Perhaps do other work here that does not require mutex lock */
101
102     }
103
104     exit(EXIT_SUCCESS);
105 }
