/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 30 */

 1 #include <time.h>
 2 #include <pthread.h>
 3 #include "tlpi_hdr.h"
 4
 5 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 6
 7 static int avail = 0;
 8
 9 static void *
10 threadFunc(void *arg)
11 {
12     int cnt = atoi((char *) arg);
13     int s, j;
14
15     for (j = 0; j < cnt; j++) {
16         sleep(1);
17
18         /* Code to produce a unit omitted */
19
20         s = pthread_mutex_lock(&mtx);
21         if (s != 0)
22             errExitEN(s, "pthread_mutex_lock");
23
24         avail++;        /* Let consumer know another unit is available */
25
26         s = pthread_mutex_unlock(&mtx);
27         if (s != 0)
28             errExitEN(s, "pthread_mutex_unlock");
29     }
30
31     return NULL;
32 }
33
34 int
35 main(int argc, char *argv[])
36 {
37     pthread_t tid;
38     int s, j;
39     int totRequired;            /* Total number of units that all
40                                    threads will produce */
41     int numConsumed;            /* Total units so far consumed */
42     Boolean done;
43     time_t t;
44
45     t = time(NULL);
46
47     /* Create all threads */
48
49     totRequired = 0;
50     for (j = 1; j < argc; j++) {
51         totRequired += atoi(argv[j]);
52
53         s = pthread_create(&tid, NULL, threadFunc, argv[j]);
54         if (s != 0)
55             errExitEN(s, "pthread_create");
56     }
57
58     /* Use a polling loop to check for available units */
59
60     numConsumed = 0;
61     done = FALSE;
62
63     for (;;) {
64         s = pthread_mutex_lock(&mtx);
65         if (s != 0)
66             errExitEN(s, "pthread_mutex_lock");
67
68         while (avail > 0) {             /* Consume all available units */
69
70             /* Do something with produced unit */
71
72             numConsumed ++;
73             avail--;
74             printf("T=%ld: numConsumed=%d\n", (long) (time(NULL) - t),
75                     numConsumed);
76
77             done = numConsumed >= totRequired;
78         }
79
80         s = pthread_mutex_unlock(&mtx);
81         if (s != 0)
82             errExitEN(s, "pthread_mutex_unlock");
83
84         if (done)
85             break;
86
87         /* Perhaps do other work here that does not require mutex lock */
88
89     }
90
91     exit(EXIT_SUCCESS);
92 }
