/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 32-2 */

 1 #include <pthread.h>
 2 #include "tlpi_hdr.h"
 3
 4 static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
 5 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 6 static int glob = 0;                    /* Predicate variable */
 7
 8 static void     /* Free memory pointed to by 'arg' and unlock mutex */
 9 cleanupHandler(void *arg)
10 {
11     int s;
12
13     printf("cleanup: freeing block at %p\n", arg);
14     free(arg);
15
16     printf("cleanup: unlocking mutex\n");
17     s = pthread_mutex_unlock(&mtx);
18     if (s != 0)
19         errExitEN(s, "pthread_mutex_unlock");
20 }
21
22 static void *
23 threadFunc(void *arg)
24 {
25     int s;
26     void *buf = NULL;                   /* Buffer allocated by thread */
27
28     buf = malloc(0x10000);              /* Not a cancellation point */
29     printf("thread:  allocated memory at %p\n", buf);
30
31     s = pthread_mutex_lock(&mtx);       /* Not a cancellation point */
32     if (s != 0)
33         errExitEN(s, "pthread_mutex_lock");
34
35     pthread_cleanup_push(cleanupHandler, buf);
36
37     while (glob == 0) {
38         s = pthread_cond_wait(&cond, &mtx);     /* A cancellation point */
39         if (s != 0)
40             errExitEN(s, "pthread_cond_wait");
41     }
42
43     printf("thread:  condition wait loop completed\n");
44     pthread_cleanup_pop(1);             /* Executes cleanup handler */
45     return NULL;
46 }
47
48 int
49 main(int argc, char *argv[])
50 {
51     pthread_t thr;
52     void *res;
53     int s;
54
55     s = pthread_create(&thr, NULL, threadFunc, NULL);
56     if (s != 0)
57         errExitEN(s, "pthread_create");
58
59     sleep(2);                   /* Give thread a chance to get started */
60
61     if (argc == 1) {            /* Cancel thread */
62         printf("main:    about to cancel thread\n");
63         s = pthread_cancel(thr);
64         if (s != 0)
65             errExitEN(s, "pthread_cancel");
66
67     } else {                    /* Signal condition variable */
68         printf("main:    about to signal condition variable\n");
69         glob = 1;
70         s = pthread_cond_signal(&cond);
71         if (s != 0)
72             errExitEN(s, "pthread_cond_signal");
73     }
74
75     s = pthread_join(thr, &res);
76     if (s != 0)
77         errExitEN(s, "pthread_join");
78     if (res == PTHREAD_CANCELED)
79         printf("main:    thread was canceled\n");
80     else
81         printf("main:    thread terminated normally\n");
82
83     exit(EXIT_SUCCESS);
84 }
