/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 30-2 */

 1 #include <pthread.h>
 2 #include "tlpi_hdr.h"
 3
 4 static volatile int glob = 0;
 5 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 6
 7 static void *                   /* Loop 'arg' times incrementing 'glob' */
 8 threadFunc(void *arg)
 9 {
10     int loops = *((int *) arg);
11     int loc, j, s;
12
13     for (j = 0; j < loops; j++) {
14         s = pthread_mutex_lock(&mtx);
15         if (s != 0)
16             errExitEN(s, "pthread_mutex_lock");
17
18         loc = glob;
19         loc++;
20         glob = loc;
21
22         s = pthread_mutex_unlock(&mtx);
23         if (s != 0)
24             errExitEN(s, "pthread_mutex_unlock");
25     }
26
27     return NULL;
28 }
29
30 int
31 main(int argc, char *argv[])
32 {
33     pthread_t t1, t2;
34     int loops, s;
35
36     loops = (argc > 1) ? getInt(argv[1], GN_GT_0, "num-loops") : 10000000;
37
38     s = pthread_create(&t1, NULL, threadFunc, &loops);
39     if (s != 0)
40         errExitEN(s, "pthread_create");
41     s = pthread_create(&t2, NULL, threadFunc, &loops);
42     if (s != 0)
43         errExitEN(s, "pthread_create");
44
45     s = pthread_join(t1, NULL);
46     if (s != 0)
47         errExitEN(s, "pthread_join");
48     s = pthread_join(t2, NULL);
49     if (s != 0)
50         errExitEN(s, "pthread_join");
51
52     printf("glob = %d\n", glob);
53     exit(EXIT_SUCCESS);
54 }
