/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 10-5 */

 1 #include <sys/times.h>
 2 #include <time.h>
 3 #include "tlpi_hdr.h"
 4
 5 static void             /* Display 'msg' and process times */
 6 displayProcessTimes(const char *msg)
 7 {
 8     struct tms t;
 9     clock_t clockTime;
10     static long clockTicks = 0;
11
12     if (msg != NULL)
13         printf("%s", msg);
14
15     if (clockTicks == 0) {      /* Fetch clock ticks on first call */
16         clockTicks = sysconf(_SC_CLK_TCK);
17         if (clockTicks == -1)
18             errExit("sysconf");
19     }
20
21     clockTime = clock();
22     if (clockTime == -1)
23         errExit("clock");
24
25     printf("        clock() returns: %ld clocks-per-sec (%.2f secs)\n",
26             (long) clockTime, (double) clockTime / CLOCKS_PER_SEC);
27
28     if (times(&t) == -1)
29         errExit("times");
30     printf("        times() yields: user CPU=%.2f; system CPU: %.2f\n",
31             (double) t.tms_utime / clockTicks,
32             (double) t.tms_stime / clockTicks);
33 }
34
35 int
36 main(int argc, char *argv[])
37 {
38     int numCalls, j;
39
40     printf("CLOCKS_PER_SEC=%ld  sysconf(_SC_CLK_TCK)=%ld\n\n",
41             (long) CLOCKS_PER_SEC, sysconf(_SC_CLK_TCK));
42
43     displayProcessTimes("At program start:\n");
44
45     numCalls = (argc > 1) ? getInt(argv[1], GN_GT_0, "num-calls") : 100000000;
46     for (j = 0; j < numCalls; j++)
47         (void) getppid();
48
49     displayProcessTimes("After getppid() loop:\n");
50
51     exit(EXIT_SUCCESS);
52 }
