/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 23-5 */

 1 #define _POSIX_C_SOURCE 199309
 2 #include <signal.h>
 3 #include <time.h>
 4 #include "curr_time.h"                  /* Declares currTime() */
 5 #include "itimerspec_from_str.h"        /* Declares itimerspecFromStr() */
 6 #include "tlpi_hdr.h"
 7
 8 #define TIMER_SIG SIGRTMAX              /* Our timer notification signal */
 9
10 static void
11 handler(int sig, siginfo_t *si, void *uc)
12 {
13     timer_t *tidptr;
14
15     tidptr = si->si_value.sival_ptr;
16
17     /* UNSAFE: This handler uses non-async-signal-safe functions
18        (printf(); see Section 21.1.2) */
19
20     printf("[%s] Got signal %d\n", currTime("%T"), sig);
21     printf("    *sival_ptr         = %ld\n", (long) *tidptr);
22     printf("    timer_getoverrun() = %d\n", timer_getoverrun(*tidptr));
23 }
24
25 int
26 main(int argc, char *argv[])
27 {
28     struct itimerspec ts;
29     struct sigaction  sa;
30     struct sigevent   sev;
31     timer_t *tidlist;
32     int j;
33
34     if (argc < 2)
35         usageErr("%s secs[/nsecs][:int-secs[/int-nsecs]]...\n", argv[0]);
36
37     tidlist = calloc(argc - 1, sizeof(timer_t));
38     if (tidlist == NULL)
39         errExit("malloc");
40
41     /* Establish handler for notification signal */
42
43     sa.sa_flags = SA_SIGINFO;
44     sa.sa_sigaction = handler;
45     sigemptyset(&sa.sa_mask);
46     if (sigaction(TIMER_SIG, &sa, NULL) == -1)
47         errExit("sigaction");
48
49     /* Create and start one timer for each command-line argument */
50
51     sev.sigev_notify = SIGEV_SIGNAL;    /* Notify via signal */
52     sev.sigev_signo = TIMER_SIG;        /* Notify using this signal */
53
54     for (j = 0; j < argc - 1; j++) {
55         itimerspecFromStr(argv[j + 1], &ts);
56
57         sev.sigev_value.sival_ptr = &tidlist[j];
58                 /* Allows handler to get ID of this timer */
59
60         if (timer_create(CLOCK_REALTIME, &sev, &tidlist[j]) == -1)
61             errExit("timer_create");
62         printf("Timer ID: %ld (%s)\n", (long) tidlist[j], argv[j + 1]);
63
64         if (timer_settime(tidlist[j], 0, &ts, NULL) == -1)
65             errExit("timer_settime");
66     }
67
68     for (;;)                            /* Wait for incoming timer signals */
69         pause();
70 }
