/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 23-2 */

 1 #define _XOPEN_SOURCE 600
 2 #include <sys/time.h>
 3 #include <time.h>
 4 #include <signal.h>
 5 #include "tlpi_hdr.h"
 6
 7 static void
 8 sigintHandler(int sig)
 9 {
10     return;                             /* Just interrupt clock_nanosleep() */
11 }
12
13 int
14 main(int argc, char *argv[])
15 {
16     struct timeval start, finish;
17     struct timespec request, remain;
18     struct sigaction sa;
19     int s, flags;
20
21     if (argc < 3 || strcmp(argv[1], "--help") == 0)
22         usageErr("%s secs nanosecs [a]\n", argv[0]);
23
24     /* Allow SIGINT handler to interrupt clock_nanosleep() */
25
26     sigemptyset(&sa.sa_mask);
27     sa.sa_flags = 0;
28     sa.sa_handler = sigintHandler;
29     if (sigaction(SIGINT, &sa, NULL) == -1)
30         errExit("sigaction");
31
32     /* If more than three command-line arguments, use TIMER_ABSTIME flag */
33
34     flags = (argc > 3) ? TIMER_ABSTIME : 0;
35
36     if (flags == TIMER_ABSTIME) {
37         if (clock_gettime(CLOCK_REALTIME, &request) == -1)
38             errExit("clock_gettime");
39         printf("Initial CLOCK_REALTIME value: %ld.%09ld\n",
40                 (long) request.tv_sec, (long) request.tv_nsec);
41
42         request.tv_sec  += getLong(argv[1], 0, "secs");
43         request.tv_nsec += getLong(argv[2], 0, "nanosecs");
44         if (request.tv_nsec >= 1000000000) {
45             request.tv_sec += request.tv_nsec / 1000000000;
46             request.tv_nsec %= 1000000000;
47         }
48
49     } else {                    /* Relative sleep */
50         request.tv_sec  = getLong(argv[1], 0, "secs");
51         request.tv_nsec = getLong(argv[2], 0, "nanosecs");
52     }
53
54     if (gettimeofday(&start, NULL) == -1)
55         errExit("gettimeofday");
56
57     for (;;) {
58         s = clock_nanosleep(CLOCK_REALTIME, flags, &request, &remain);
59         if (s != 0 && s != EINTR)
60             errExitEN(s, "clock_nanosleep");
61
62         if (s == EINTR)
63             printf("Interrupted... ");
64
65         if (gettimeofday(&finish, NULL) == -1)
66             errExit("gettimeofday");
67         printf("Slept: %.6f secs", finish.tv_sec - start.tv_sec +
68                         (finish.tv_usec - start.tv_usec) / 1000000.0);
69
70         if (s == 0)
71             break;                      /* sleep completed */
72
73         if (flags != TIMER_ABSTIME) {
74             printf("... Remaining: %ld.%09ld",
75                     (long) remain.tv_sec, remain.tv_nsec);
76
77             request = remain;
78         }
79
80         printf("... Restarting\n");
81     }
82
83     printf("\nSleep complete\n");
84     exit(EXIT_SUCCESS);
85 }
