/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 23-2 */

 1 #include <signal.h>
 2 #include "tlpi_hdr.h"
 3
 4 #define BUF_SIZE 200
 5
 6 static void     /* SIGALRM handler: interrupts blocked system call */
 7 handler(int sig)
 8 {
 9     printf("Caught signal\n");          /* UNSAFE (see Section 21.1.2) */
10 }
11
12 int
13 main(int argc, char *argv[])
14 {
15     struct sigaction sa;
16     char buf[BUF_SIZE];
17     ssize_t numRead;
18     int savedErrno;
19
20     if (argc > 1 && strcmp(argv[1], "--help") == 0)
21         usageErr("%s [num-secs [restart-flag]]\n", argv[0]);
22
23     /* Set up handler for SIGALRM. Allow system calls to be interrupted,
24        unless second command-line argument was supplied. */
25
26     sa.sa_flags = (argc > 2) ? SA_RESTART : 0;
27     sigemptyset(&sa.sa_mask);
28     sa.sa_handler = handler;
29     if (sigaction(SIGALRM, &sa, NULL) == -1)
30         errExit("sigaction");
31
32     alarm((argc > 1) ? getInt(argv[1], GN_NONNEG, "num-secs") : 10);
33
34     numRead = read(STDIN_FILENO, buf, BUF_SIZE - 1);
35
36     savedErrno = errno;                 /* In case alarm() changes it */
37     alarm(0);                           /* Ensure timer is turned off */
38     errno = savedErrno;
39
40     /* Determine result of read() */
41
42     if (numRead == -1) {
43         if (errno == EINTR)
44             printf("Read timed out\n");
45         else
46             errMsg("read");
47     } else {
48         printf("Successful read (%ld bytes): %.*s",
49                 (long) numRead, (int) numRead, buf);
50     }
51
52     exit(EXIT_SUCCESS);
53 }
