 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 62-3 */
12
13  1 #include <termios.h>
14  2 #include <unistd.h>
15  3 #include "tty_functions.h"              /* Declares functions defined here */
16  4
17  5 /* Place terminal referred to by 'fd' in cbreak mode (noncanonical mode
18  6    with echoing turned off). This function assumes that the terminal is
19  7    currently in cooked mode (i.e., we shouldn't call it if the terminal
20  8    is currently in raw mode, since it does not undo all of the changes
21  9    made by the ttySetRaw() function below). Return 0 on success, or -1
22 10    on error. If 'prevTermios' is non-NULL, then use the buffer to which
23 11    it points to return the previous terminal settings. */
24 12
25 13 int
26 14 ttySetCbreak(int fd, struct termios *prevTermios)
27 15 {
28 16     struct termios t;
29 17
30 18     if (tcgetattr(fd, &t) == -1)
31 19         return -1;
32 20
33 21     if (prevTermios != NULL)
34 22         *prevTermios = t;
35 23
36 24     t.c_lflag &= ~(ICANON | ECHO);
37 25     t.c_lflag |= ISIG;
38 26
39 27     t.c_iflag &= ~ICRNL;
40 28
41 29     t.c_cc[VMIN] = 1;                   /* Character-at-a-time input */
42 30     t.c_cc[VTIME] = 0;                  /* with blocking */
43 31
44 32     if (tcsetattr(fd, TCSAFLUSH, &t) == -1)
45 33         return -1;
46 34
47 35     return 0;
48 36 }
49 37
50 38 /* Place terminal referred to by 'fd' in raw mode (noncanonical mode
51 39    with all input and output processing disabled). Return 0 on success,
52 40    or -1 on error. If 'prevTermios' is non-NULL, then use the buffer to
53 41    which it points to return the previous terminal settings. */
54 42
55 43 int
56 44 ttySetRaw(int fd, struct termios *prevTermios)
57 45 {
58 46     struct termios t;
59 47
60 48     if (tcgetattr(fd, &t) == -1)
61 49         return -1;
62 50
63 51     if (prevTermios != NULL)
64 52         *prevTermios = t;
65 53
66 54     t.c_lflag &= ~(ICANON | ISIG | IEXTEN | ECHO);
67 55                         /* Noncanonical mode, disable signals, extended
68 56                            input processing, and echoing */
69 57
70 58     t.c_iflag &= ~(BRKINT | ICRNL | IGNBRK | IGNCR | INLCR |
71 59                       INPCK | ISTRIP | IXON | PARMRK);
72 60                         /* Disable special handling of CR, NL, and BREAK.
73 61                            No 8th-bit stripping or parity error handling.
74 62                            Disable START/STOP output flow control. */
75 63
76 64     t.c_oflag &= ~OPOST;                /* Disable all output processing */
77 65
78 66     t.c_cc[VMIN] = 1;                   /* Character-at-a-time input */
79 67     t.c_cc[VTIME] = 0;                  /* with blocking */
80 68
81 69     if (tcsetattr(fd, TCSAFLUSH, &t) == -1)
82 70         return -1;
83 71
84 72     return 0;
85 73 }
