/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 62-2 */

  1 #include <dirent.h>
  2 #include <sys/stat.h>
  3 #include <unistd.h>
  4 #include <stdlib.h>
  5 #include <string.h>
  6 #include <stdio.h>
  7
  8 /* Helper function for ttyname(). We do most of the real work here.
  9    Look in 'devDir' for the terminal device name corresponding to the
 10    stat structure given in 'fdStat' (which the caller must ensure
 11    derives from a terminal device).
 12
 13    Return the device name (as a statically-allocated) string if
 14    found, or NULL if the device is not found or an error occurs */
 15
 16 static char *
 17 ttynameCheckDir(const struct stat *fdStat, const char *devDir)
 18 {
 19     DIR *dirh;
 20     struct dirent *dent;
 21     static char *ttyPath;               /* Currently checked entry; also used
 22                                            to return tty name, if found */
 23     static int ttyLen = 0;              /* Length of ttyPath */
 24     struct stat devStat;                /* stat entry for ttyPath */
 25     int found;                          /* True if we find device entry */
 26     int requiredLen;
 27
 28     if (ttyLen == 0) {                  /* First call - allocate ttyPath */
 29         ttyPath = malloc(50);
 30         if (ttyPath == NULL)
 31             return NULL;
 32         ttyLen = 50;
 33     }
 34
 35     dirh = opendir(devDir);
 36     if (dirh == NULL)
 37         return NULL;
 38
 39     /* We walk through each file in /dev looking for an entry whose
 40        device ID (st_rdev) matches the device ID corresponding to 'fd'.
 41        To do this, we construct a pathname for each entry in /dev, and
 42        then call stat() on that pathname. This is somewhat expensive.
 43        The glibc implementation of ttyname() performs an optimization:
 44        it performs a first pass of the entries in /dev, performing
 45        a stat() call only if fdStat->st_ino == dent->d_ino. This speeds
 46        the search (since many calls to stat() are avoided), but is not
 47        guaranteed to work in every case (e.g., if there are symbolic
 48        links in /dev). Therefore, if the first pass fails to find a
 49        matching device, glibc's ttyname() performs a second pass
 50        without the st_ino check (i.e., like we do below). */
 51
 52     found = 0;
 53     while ((dent = readdir(dirh)) != NULL) {
 54         requiredLen = strlen(devDir) + 1 + strlen(dent->d_name) + 1;
 55
 56         if (requiredLen > ttyLen) {     /* Resize ttyPath if required */
 57             ttyPath = realloc(ttyPath, requiredLen);
 58             if (ttyPath == NULL)
 59                 break;
 60             ttyLen = requiredLen;
 61         }
 62
 63         snprintf(ttyPath, ttyLen, "%s/%s", devDir, dent->d_name);
 64
 65         if (stat(ttyPath, &devStat) == -1)
 66             continue;                   /* Ignore unstat-able entries */
 67
 68         if (S_ISCHR(devStat.st_mode) &&
 69                 fdStat->st_rdev == devStat.st_rdev) {
 70             found = 1;
 71             break;
 72         }
 73     }
 74
 75     closedir(dirh);
 76
 77     return found ? ttyPath : NULL;
 78 }
 79
 80 /* Return the name of the terminal associated with 'fd' (in a
 81    statically-allocated string that is overwritten on subsequent
 82    calls), or NULL if it is not a terminal or an error occurs */
 83
 84 char *
 85 ttyname(int fd)
 86 {
 87     char *d;
 88     struct stat fdStat;                 /* stat entry for fd */
 89
 90     if (!isatty(fd))                    /* Is fd even a terminal? */
 91         return NULL;
 92
 93     if (fstat(fd, &fdStat) == -1)
 94         return NULL;
 95
 96     if (!S_ISCHR(fdStat.st_mode))       /* Is fd even a character device? */
 97         return NULL;
 98
 99     /* First check for a pseudoterminal entry in the /dev/pts
100        directory. If that fails, try looking in /dev.
101        (We check the directories in this order for efficiency:
102        /dev/pts is small, and will contain the required device if
103        fd refers to an X-terminal or similar.)  */
104
105     d = ttynameCheckDir(&fdStat, "/dev/pts");
106     return (d != NULL) ? d : ttynameCheckDir(&fdStat, "/dev");
107 }
