 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 8-1 */
12
13  1 #include <pwd.h>
14  2 #include <grp.h>
15  3 #include <ctype.h>
16  4 #include "ugid_functions.h"     /* Declares functions defined here */
17  5
18  6 char *          /* Return name corresponding to 'uid', or NULL on error */
19  7 userNameFromId(uid_t uid)
20  8 {
21  9     struct passwd *pwd;
22 10
23 11     pwd = getpwuid(uid);
24 12     return (pwd == NULL) ? NULL : pwd->pw_name;
25 13 }
26 14
27 15 uid_t           /* Return UID corresponding to 'name', or -1 on error */
28 16 userIdFromName(const char *name)
29 17 {
30 18     struct passwd *pwd;
31 19     uid_t u;
32 20     char *endptr;
33 21
34 22     if (name == NULL || *name == '\0')  /* On NULL or empty string */
35 23         return -1;                      /* return an error */
36 24
37 25     u = strtol(name, &endptr, 10);      /* As a convenience to caller */
38 26     if (*endptr == '\0')                /* allow a numeric string */
39 27         return u;
40 28
41 29     pwd = getpwnam(name);
42 30     if (pwd == NULL)
43 31         return -1;
44 32
45 33     return pwd->pw_uid;
46 34 }
47 35
48 36 char *          /* Return name corresponding to 'gid', or NULL on error */
49 37 groupNameFromId(gid_t gid)
50 38 {
51 39     struct group *grp;
52 40
53 41     grp = getgrgid(gid);
54 42     return (grp == NULL) ? NULL : grp->gr_name;
55 43 }
56 44
57 45 gid_t           /* Return GID corresponding to 'name', or -1 on error */
58 46 groupIdFromName(const char *name)
59 47 {
60 48     struct group *grp;
61 49     gid_t g;
62 50     char *endptr;
63 51
64 52     if (name == NULL || *name == '\0')  /* On NULL or empty string */
65 53         return -1;                      /* return an error */
66 54
67 55     g = strtol(name, &endptr, 10);      /* As a convenience to caller */
68 56     if (*endptr == '\0')                /* allow a numeric string */
69 57         return g;
70 58
71 59     grp = getgrnam(name);
72 60     if (grp == NULL)
73 61         return -1;
74 62
75 63     return grp->gr_gid;
76 64 }
