/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 16-1 */

 1 #include <sys/xattr.h>
 2 #include "tlpi_hdr.h"
 3
 4 #define XATTR_SIZE 10000
 5
 6 static void
 7 usageError(char *progName)
 8 {
 9     fprintf(stderr, "Usage: %s [-x] file...\n", progName);
10     exit(EXIT_FAILURE);
11 }
12
13 int
14 main(int argc, char *argv[])
15 {
16     char list[XATTR_SIZE], value[XATTR_SIZE];
17     ssize_t listLen, valueLen;
18     int ns, j, k, opt;
19     Boolean hexDisplay;
20
21     hexDisplay = 0;
22     while ((opt = getopt(argc, argv, "x")) != -1) {
23         switch (opt) {
24         case 'x': hexDisplay = 1;       break;
25         case '?': usageError(argv[0]);
26         }
27     }
28
29     if (optind >= argc)
30         usageError(argv[0]);
31
32     for (j = optind; j < argc; j++) {
33         listLen = listxattr(argv[j], list, XATTR_SIZE);
34         if (listLen == -1)
35             errExit("listxattr");
36
37         printf("%s:\n", argv[j]);
38
39         /* Loop through all EA names, displaying name + value */
40
41         for (ns = 0; ns < listLen; ns += strlen(&list[ns]) + 1) {
42             printf("        name=%s; ", &list[ns]);
43
44             valueLen = getxattr(argv[j], &list[ns], value, XATTR_SIZE);
45             if (valueLen == -1) {
46                 printf("couldn't get value");
47             } else if (!hexDisplay) {
48                 printf("value=%.*s", (int) valueLen, value);
49             } else {
50                 printf("value=");
51                 for (k = 0; k < valueLen; k++)
52                     printf("%02x ", (unsigned int) value[k]);
53             }
54
55             printf("\n");
56         }
57
58         printf("\n");
59     }
60
61     exit(EXIT_SUCCESS);
62 }
